package kerberos.messages;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.KerberosRuntimeException;
import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.packets.PacketException;
import kerberos.packets.PacketReader;

/**
 * Oberklasse von spezialisierten Klassen um eine einheitliche Verarbeitung zu ermoeglichen.
 * 
 * @author seifer_s, stier_j
 *	
 */
public abstract class Message {

	public static final int M_FIRST = 0;
	// diese Message muss immer die Nummer der ersten haben.
	public static final int M_NO_MESSAGE = 0;
	public static final int M_KDC_AS_REQ = 1; // Anfrage an den AuthentifizierungsService des KDCs TODO Simon und Jendrik: alle Felder dokumentieren mit javadoc
	public static final int M_KDC_AS_REP = 2;
	public static final int M_KDC_TGS_REQ = 3;
	public static final int M_KDC_TGS_REP = 4;
	public static final int M_ERROR = 6;
	public static final int M_AP_REQ = 5;
	public static final int M_AP_REP = 7;
	public static final int M_AP_REQ2 = 8;
	/** M_TYP_UNBEKANNT = Message-Typ unbekannt, weil er noch gar nicht gelesen wurde */
	public static final int M_TYP_UNBEKANNT = 9;
	public static final int M_LAST = 9;
	// diese Message muss immer die Nummer der letzten haben.

	/**
	 * Gibt zu dem bergebenen MessageType (Zahl) eine Stringreprsentation zurck
	 * oder die Zahl als String, wenn der Zahl keine Fehlermeldungsnummer zugeordnet werden konnte 
	 */
	public static String getString(int messageType)
	{
		switch(messageType)
		{
			case M_NO_MESSAGE: return "NO_MESSAGE";
			case M_KDC_AS_REQ: return "KDC_AS_REQ";
			case M_KDC_AS_REP: return "KDC_AS_REP";
			case M_KDC_TGS_REQ: return "KDC_TGS_REQ";
			case M_KDC_TGS_REP: return "KDC_TGS_REP";
			case M_AP_REQ: return "AP_REQ";
			case M_AP_REP: return "AP_REP";
			case M_AP_REQ2: return "AP_REQ2";
			case M_ERROR: return "ERROR";
		}
		return ""+messageType;
	}

	protected Message()
	{
	}

	public static Message createMessage(byte[] message, byte[] schluessel)
		throws PacketException, PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException
	{
		PacketReader pr = new PacketReader(message);
		int messageTyp = pr.getMessageType();
		switch(messageTyp)
		{
			case M_KDC_AS_REQ:  return new M_KDC_AS_REQ(message); 						
			case M_KDC_AS_REP:  return new M_KDC_AS_REP(message, schluessel);						
			case M_KDC_TGS_REQ: return new M_KDC_TGS_REQ(message, schluessel);						
			case M_KDC_TGS_REP: return new M_KDC_TGS_REP(message, schluessel);						
			case M_AP_REQ: return new M_AP_REQ(message, schluessel);	
			case M_AP_REQ2: return new M_AP_REQ2(message,schluessel);					
			case M_AP_REP: return new M_AP_REP(message, schluessel);
			case M_ERROR:  return new M_ERROR(message);
			default:
			{
				throw new UngltigerMessageTypException(messageTyp);
			}
		}
	}

	/**
	 * Wirft eine NonceFalschException, wenn der NonceWert der Message ungleich dem bergebenen Nonce-Wert ist. 
	 * @param nonce
	 * @throws NonceFalschException
	 */
	public abstract void checkNonce(long nonce) throws NonceFalschException;
	
	/**
	 * Gibt den Message-Typ zurck
	 * @return
	 */
	public abstract int getType();	
	
	/**
	 * Testet, ob die Zeit in diesem Paket in Ordnung ist.
	 * @param message Nachricht, dessen Zeit berprft werden soll
	 * @return Gibt die Fehlermessage als Byte-Array zurck, oder null, wenn alles OK ist 
	 * @throws KerberosRuntimeException Wenn es einen Fehler gab, der eigentlich nicht auftreten sollte.
	 */
	public static byte[] checkZeit(MessageWithTime message) throws KerberosRuntimeException
	{
		try
		{
			long aktuelleZeit = System.currentTimeMillis();
			if(aktuelleZeit >= message.getEndzeit()) // Ticket abgelaufen?
			{
				return MessageBuilder.currentBuilder().buildError(message.getType(), kerberos.messages.M_ERROR.ERR_TKT_EXPIRED);
			}
			if(message.getStartzeit() > message.getEndzeit()) // Das Ticket kann nie gltig sein
			{
				return MessageBuilder.currentBuilder().buildError(message.getType(), kerberos.messages.M_ERROR.ERR_NEVER_VALID);
			}
			if(aktuelleZeit <= message.getStartzeit()) // Ticket schon gltig?
			{
				return MessageBuilder.currentBuilder().buildError(message.getType(), kerberos.messages.M_ERROR.ERR_TKT_NYV);
			}
			return null;
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
	}
}
