/*
 * Created on 07.10.2003
 */
package kerberos.net;

import java.io.IOException;
import util.logger.*;
import kerberos.exceptions.*;

/**
 * Diese Klasse Dient auf der Clientseite dazu, die KerberosSockets zu erzeugen.
 * Die Factory speichert auch das Ticket-Granting-Ticket.
 * @author Stephan Koops
 */
public interface KerberosClientSocketFactory
{
	/**
	 * Baut eine Verbindung zum Ziel-Server auf. Die Verbindung ist ist mit Kerberos verschlsselt. 
	 * Die Factory darf vorher nicht mit <a href="#destroy()">destroy()</a> zerstrt worden sein. 
	 * @param hostname Hostname (oder IP-Adresse) des Ziel-Servers
	 * @param port Port auf dem Ziel-Server
	 * @param zieldienst Name des Ziel-Dienstes im KDC
	 * @throws IOException wenn es beim Socket erzeugen oder bei der Anmeldung
	 * beim KDC Netzwerk-Probleme gab.
	 * @throws kerberos.exceptions.KerberosException Alle hier geworfenen Exceptions 
	 * (auer IOException) sind von der KerberosException abgeleitet. Die Bedeutung der
	 * Exception ist im Javadoc zu den entsprechenden Klassen erlutert.
	 * @see #destroy()
	 */
	public KerberosSocket connect(String hostname, int port, String zieldienst)
		throws ArrayZuLangException, IOException, FactoryDestroyedException, PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, NonceFalschException, UnerwarteteNachrichtException, UnbekannterParticipalException, ServerdienstNichtVorhandenException, TicketUngueltigException;

	/**
	 * Zerstrt die alle sensiblen Daten dieser Factory, also Tickets und Schlssel, so dass diese garantiert 
	 * aus dem RAM gelscht werden. Die Factory ist danach nicht mehr nutzbar, und wirft bei Benutzung der 
	 * Methode <a href="#connect(java.lang.String, int, java.lang.String)">connect(String, int, String)</a>
	 * eine FactoryDestroyedException.
	 * @see #connect(String, int, String)
	 * @see FactoryDestroyedException
	 */
	public void destroy();

	/**
	 * Gibt die maximal erlaubte Lnge fr zu empfangende Arrays 
	 * der erzeugten KerberosSockets zurck.<br>Die maximale Lnge lsst sich begrenzen, damit kein
	 * Angreifer vorgibt, dass ein 1 Giga-Byte groes Array eingelesen weren soll. Dann wrde nhmlich
	 * ein entsprechend groes Array erzeugt werde, was die meisten Rechner mit einem
	 * OutOfMemoryError quittieren wrden.
	 * @see OutOfMemoryError
	 */
	public int getMaxArrayLength();

	/**
	 * Setzt die maximal erlaubte Lnge fr zu empfangende Nachrichten
	 * @see #getMaxArrayLength()
	 */
	public void setMaxArrayLength(int i) throws ZahlNichtPositivException;
	
	/**
	 * Setzt den Logger, der ab sofort fr die neu erstellten KerberosSockets
	 * verwendet wird. Fr schon erzeugte Sockets msste der Logger dort explizit
	 * verndert werden. Deshalb sollte der Logger einmal beim Starten der Anwendung gesetzt werden.
	 * @param logger Wenn der Logger null ist, werden die Sachen nicht mehr geloggt.
	 */
	public void setLogger(Logger logger);

	/**
	 * Gibt den aktuell verwendeten Logger zurck
	 * @see #setLogger(Logger)
	 */
	public Logger getLogger();
}

