/*
 * Created on 07.10.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.Socket;

import sun.net.ConnectionResetException;
import util.NetUtil;
import util.logger.*;

import kerberos.exceptions.ArrayZuLangException;
import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.FactoryDestroyedException;
import kerberos.exceptions.KerberosRuntimeException;
import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortDestroyedException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.ServerdienstNichtVorhandenException;
import kerberos.exceptions.TicketUngueltigException;
import kerberos.exceptions.UnbekannterParticipalException;
import kerberos.exceptions.UnerwarteteNachrichtException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.messages.M_AP_REP;
import kerberos.messages.M_ERROR;
import kerberos.messages.M_KDC_AS_REP;
import kerberos.messages.M_KDC_TGS_REP;
import kerberos.messages.Mess_mem_Nonce;
import kerberos.messages.Message;
import kerberos.messages.MessageBuilder;
import kerberos.exceptions.ZahlNichtPositivException;
import kerberos.packets.PacketException;
import kerberos.server.KDCServer;

/**
 * Diese Klasse dient auf der Clientseite dazu, die KerberosSockets zu erzeugen.
 * Die Factory speichert auch das Ticket-Granting-Ticket.
 * @author Stephan Koops
 */
public class KerberosClientSocketFactoryImpl implements KerberosClientSocketFactory
{
	/** Speichert die Standard-Maximallnge fr zu empfangende Arrays fr die ClientSocketFactory */
	public static final int ARRAY_MAX_LAENGE = 256*256;

	/** Speichert die Standard-Maximal-Array-Lnge fr die erzeugten KerberosSockets */ 	
	public static final int STANDARD_SOCKET_ARRAY_MAX_LAENGE = 256*256;
	
	public static final int kdcAuthPort = KDCServer.PORT;

	public static final boolean debug = false;

	/** Speichert die maximale Array-Lnge fr zu empfangene Daten */
	private int maxArrayLength = STANDARD_SOCKET_ARRAY_MAX_LAENGE;

	private byte[] ticketGrantingTicket;
	
	private Logger logger = NullLogger.get();
	
	/**
	 * Sitzungssschlssel von diesem Client und dem Ticket-Granting-Service
	 * = K_{C,TGS}
	 */
	private byte[] schlssel_C_TGS;
	
	/**
	 * Dieses flag ist auf true, wenn die Factory nicht mehr verwendet werden kann,
	 * weil sie zerstrt wurde. Das Flag muss abgefragt werden, bevor das TicketGratingTicket 
	 * oder der Schlssel{C,TGS} verwendet wird. Da im Prinzip mehrere Threads die Factory verwenden
	 * knnen, muss ein die Abfrage des Threads und der genannten Werte synchronized erfolgen.
	 */
	private boolean zerstrt = false;
	
	protected String username;

	protected Socket socketToKDC;

	/**
	 * Dieser Konstruktor erzeugt eine Factory, mit deren Hilfe (mit der Methode 
	 * <a href="#connect(String,int,String)">connect(...)</a> ) Verbindungen zu 
	 * beliebig vielen Ziel-Servern aufgebaut werden knnen.<br>
	 * Der Konstruktor sorgt fr die Authentifizierung beim KDC und speichert auch 
	 * das TGS-Ticket.
	 * @param username
	 * @param passwort Passwort des Benutzers. Das Passwort wird whrend der 
	 * Verarbeitung zerstrt.
	 * @param kdcHostnameOrIP
	 * @throws IOException wenn es beim Socket erzeugen oder bei der Anmeldung
	 *         beim KDC Netzwerk-Probleme gab.
	 * @throws PasswortFalschException Wenn das angegebene Passwort falsch ist.
	 * @throws KerberosException Wenn es ein Authentifizierungsproblem o.. gab.
	 * Wird auch geworfen, wenn es ein Problem beim Erzeugen des internen Paketes gab.
	 * @throws UnerwarteteNachrichtException Wird geworfen, wenn vom Server eine Nachricht 
	 * kommt, mit dem die KerberosClientSocketFactory nichts anfangen kann. Wenn der KDC
	 * korrekt arbeitet, kommt dies aber nicht vor
	 * @throws RuntimeException Wird geworfen, wenn es Problem beim Schreiben auf einen
	 * ByteArrayOutputStream gab.
	 */
	public KerberosClientSocketFactoryImpl(String username, Passwort passwort, String kdcHostnameOrIP)
		throws UnerwarteteNachrichtException, NonceFalschException, PasswortDestroyedException, UnbekannterParticipalException, TicketUngueltigException, ConnectionResetException, PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, ArrayZuLangException, IOException
	{
		this.username = username;
		byte[] schlssel = passwort.getSchlssel(); // Hier wird das Passwort automatisch zerstrt
		Mess_mem_Nonce authLogin;
		try
		{
			authLogin = MessageBuilder.currentBuilder().buildKdcAuthLogin(username);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
		long sendNonce = authLogin.nonce;
		if(debug)
			System.out.println(
				"KerberosClientSocketFactory.<init> Verbindung aufbauen zu KDC "
				+ kdcHostnameOrIP + ", Port " + kdcAuthPort);
		this.socketToKDC = new Socket(kdcHostnameOrIP, kdcAuthPort);
		Message antwort = sendReadKDC(authLogin.message, schlssel);
		if (antwort instanceof M_ERROR)
		{
			M_ERROR antw = (M_ERROR)antwort;
			antw.checkForReasonInvalidTicket();
			antw.checkForReasonInvalidPrincipal();
			antw.checkForReasonArrayZuLang();
			throw new UnerwarteteNachrichtException(antw);
		}
		antwort.checkNonce(sendNonce);
		if (antwort instanceof M_KDC_AS_REP)
		{
			M_KDC_AS_REP posAntw = (M_KDC_AS_REP)antwort;
			this.ticketGrantingTicket = posAntw.getTicket();
			if(debug)
				System.out.println(
				"KerberosClientSocketFactory.<init> KDC (" + kdcHostnameOrIP + ") hat Verbindung angenommen");
			this.schlssel_C_TGS = posAntw.getSchluessel_C_TGS(); // Sitzungsschlssel K_{Client,TGS} 
		}
		else
		{
			throw new UnerwarteteNachrichtException(antwort);
		}
		/*
		 * Das knnte evtl asynchron geschehen, so dass der
		 * Aufrufende mit etwas anderes weitermachen kann, bis die Factory 
		 * sich beim KDC authentifiziert hat. Die anderen Methoden msste man dann
		 * entsprechend synchronisieren.
		 * Das gleiche gilt fr den KerberosSocket.
		 */
	}

	/**
	 * Baut eine Verbindung zum Ziel-Server auf.
	 * @param hostname Hostname des Ziel-Servers
	 * @param port Ziel-Port
	 * @param zieldienst Name des Ziel-Dienstes im KDC
	 * @return Gibt einen IKerberosSocket zurck, mit dessen Hilfe mit dem Server fast wie
	 * mit einem normalen Socket kommuniziert werden kann.
	 * @throws IOException wenn es beim Socket erzeugen oder bei der Anmeldung
	 * beim KDC Netzwerk-Probleme gab.
	 * @throws RuntimeException Kann geworfen werden, wenn es ein Problem beim Schreiben auf einen
	 * ByteOutputStream gab. Dieser Fehler sollte eigentlich nicht auftreten  
	 */
	public KerberosSocket connect(String hostname, int port, String zieldienst)
		throws UnbekannterParticipalException, ConnectionResetException, FactoryDestroyedException, UnerwarteteNachrichtException, NonceFalschException, TicketUngueltigException, ServerdienstNichtVorhandenException, PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, ArrayZuLangException, IOException
	{
		if(debug)
			System.out.println(
			"KerberosClientSocketFactory.connect(): gleich Socket zu " + hostname + ":" + port + " aufbauen");
		Message antwort = connectGetSchlssel(zieldienst);
		if (antwort instanceof M_KDC_TGS_REP) // positive Antwort des TGS
		{
			return connectToZielServer((M_KDC_TGS_REP)antwort, hostname, port);
		}
		throw new UnerwarteteNachrichtException(antwort);
	}
	
	/**
	 * Diese Methode versucht, sich beim KDC den Schlssel fr den gewnschten Zieldienst zu holen.
	 * Die Antwortnachricht des Ticket-Granting-Service des KDCs wird zurckgegeben.<br>
	 * Wenn diese Antwort vom KDC eine Fehler-Nachricht ist, dann wird eine entsprechende Exception gewrofen
	 * Hinweis: Diese Methode enthlt einen synchronized-Abschnitt.
	 * @param zieldienst Name des Dienstes, fr das das Ticket geholt werden soll
	 * @param zeit ist im MessageBuilder leider nicht kommentiert
	 * @return Antwort-Nachricht des KDCs
	 */
	private Message connectGetSchlssel(String zieldienst) throws FactoryDestroyedException, UnerwarteteNachrichtException, NonceFalschException, TicketUngueltigException, ServerdienstNichtVorhandenException, ConnectionResetException, PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, ArrayZuLangException, IOException
	{
		Mess_mem_Nonce tgsLogin;
		byte[] schlssel_C_TGS; // wird hier zwischengespeichert, damit der synchronized-Teil nicht so lang ist.
		synchronized (this)
		{
			if(this.zerstrt)
				throw new FactoryDestroyedException(this.getClass());
			try
			{
				schlssel_C_TGS = this.schlssel_C_TGS;
				tgsLogin = MessageBuilder.currentBuilder().buildTGSLogin(this.ticketGrantingTicket, username, this.schlssel_C_TGS, zieldienst);
			}
			catch (PacketException pe)
			{
				throw KerberosRuntimeException.throwWith(pe);
			}
		}
		long sendNonce = tgsLogin.nonce;
		this.socketToKDC = new Socket(this.socketToKDC.getInetAddress(), kdcAuthPort);
		if(debug)
			System.out.println("KerberosClientSocketFactory.connect(): tgslogin ist abgeschickt");
		Message antwort = sendReadKDC(tgsLogin.message, schlssel_C_TGS);
		if (antwort instanceof M_ERROR)
		{
			M_ERROR fehlerantwort = (M_ERROR)antwort;
			fehlerantwort.checkForReasonInvalidTicket();
			fehlerantwort.checkForReasonInvalidZieldienst();
			throw new UnerwarteteNachrichtException(fehlerantwort);
		}
		if(debug)
			System.out.print("KerberosClientSocketFactory.connect(): tgslogin beantwortet, gleich Test des Nonce-Wertes");
		antwort.checkNonce(sendNonce);
		if(debug)
			System.out.println(" -> OK");
		return antwort;
	}

	/**
	 * Diese Methode baut eine Verbindung zum Ziel-Server auf und versucht, sich ber Kerberos zu authentifizieren.
	 * @param posTgsAntw positive Antwort des Ticket-Granting-Services des KDCs mit dem Ticket fr den Server
	 * @param socketToClient Socket, ber den die Verbindung laufen soll.
	 * @param zeit
	 * @return KerberosSocket mit der Verbindung zum Client
	 * @throws KerberosException Alle hier geworfenen Exceptions auer die 
	 * IOException erben von der Klasse KerberosException. Weiter 
	 * Informationen in den jeweiligen Exception-Klassen.
	 * @throws IOException
	 */
	private KerberosSocket connectToZielServer(M_KDC_TGS_REP posTgsAntw, String hostname, int port)
		throws PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, NonceFalschException, UnerwarteteNachrichtException, UnbekannterParticipalException, ServerdienstNichtVorhandenException, TicketUngueltigException, ConnectionResetException, ArrayZuLangException, IOException
	{
		String methode = "KerberosClientSocketFactory.connectToZielServer(..)";
		if(debug)
			System.out.println(methode+": TGS-Login erfolgreich: positive Antwort erhalten");
		byte[] serverticket = posTgsAntw.getTicket();
		byte[] schlssel_CS = posTgsAntw.getSchluessel();
		Mess_mem_Nonce serverLogin;
		if(debug)
		{
			//System.out.println(methode+" mit Serverticket="+Passwort.toZahl(serverticket));
			System.out.println(methode+" mit this.username="+this.username);
			System.out.println(methode+" mit Schlssel{C,S}="+Passwort.toZahl(schlssel_CS));
		}
		try
		{
			serverLogin = MessageBuilder.currentBuilder().buildApLogin(serverticket, this.username, schlssel_CS);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
		long serverNonce = serverLogin.nonce;
		Socket socketToClient = new Socket(hostname, port);
		if (debug)
			System.out.println("KerberosClientSocketFactory.connectToZielServer: Login beim Client: " + socketToClient);
		NetUtil.writeArray(socketToClient, serverLogin.message);
		byte[] cryptServerAntwort = NetUtil.readByteArray(socketToClient, ARRAY_MAX_LAENGE);
		Message antwortMess;
		try
		{
			if (debug)
				System.out.println("KerberosClientSocketFactory.connectToZielServer: Schlssel{C,S}="+Passwort.toZahl(schlssel_CS));
			antwortMess = MessageBuilder.currentBuilder().readMessage(cryptServerAntwort, schlssel_CS);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
		if (antwortMess instanceof M_AP_REP)
		{
			M_AP_REP posServerAntw = (M_AP_REP)antwortMess;
			posServerAntw.checkNonce(serverNonce);
			if (debug)
				System.out.println("KerberosClientSocketFactory.connectToZielServer: new KerberosSocketC(..,..," + this.username + ",..)");
			return new KerberosSocketC(socketToClient, schlssel_CS, this.username, serverticket, this.maxArrayLength, this.logger);
		}
		if(antwortMess instanceof M_ERROR)
		{
			M_ERROR errMessage = (M_ERROR)antwortMess;
			errMessage.checkForReasonInvalidPrincipal();
			errMessage.checkForReasonInvalidZieldienst();
			errMessage.checkForReasonInvalidTicket();
		}
		throw new UnerwarteteNachrichtException(posTgsAntw);
	}

	/**
	 * Schickt eine Nachricht zum KDC und gibt die Antwort zurck
	 * @param message zum KDC zu sendende Nachricht
	 * @param schlssel Schlssel, mit dem die Antwort entschlsselt werden soll.
	 * @return Empfangene Message
	 * @throws IOException, Wenn es Netzwerkprobleme gab
	 * @throws PasswortFalschException Wenn die Nachricht mit dem falschen Passwort verschlsselt wurde.
	 */
	private Message sendReadKDC(byte[] message, byte[] schlssel)
		throws PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, ConnectionResetException, ArrayZuLangException, IOException
	{
		//boolean debug = false;
		//		try { Thread.sleep(2000); } catch (InterruptedException e) { }
		if(debug)
		{
			// isConnected luft auf dem PDA nicht
			System.out.println("KerberosClientSocketFactory.sendReadKDC: socketToKDC ist connected: " + this.socketToKDC.isConnected());
			System.out.println("KerberosClientSocketFactory.sendReadKDC: socketToKDC ist closed: " + this.socketToKDC.isClosed());
		}
		NetUtil.writeArray(this.socketToKDC, message);
		if(debug)
			System.out.println("KerberosClientSocketFactory.sendReadKDC: message an KDC gesendet");
		byte[] fromAuth = NetUtil.readByteArray(socketToKDC, ARRAY_MAX_LAENGE);
		if(debug)
			System.out.println("KerberosClientSocketFactory.sendReadKDC: Antwort-Array vom KDC eingelesen");
		try
		{
			return MessageBuilder.currentBuilder().readMessage(fromAuth, schlssel);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
	}

	/**
	 * Zerstrt die Tickets, so dass diese garantiert aus dem RAM gelscht werden.
	 * Danach ist diese Socket-Factory nicht mehr nutzbar! Die Methode 
	 * <a href="#accept(String,int,String)">accept(...)</a> wirft dann eine
	 * FactoryDestroyedException
	 * @see FactoryDestroyedException
	 */
	public synchronized void destroy()
	{
		this.zerstrt = true;
		Passwort.destroyArray(this.schlssel_C_TGS);
		Passwort.destroyArray(this.ticketGrantingTicket);
	}

	/**
	 * @see java.lang.Object#finalize()
	 */
	public void finalize() throws Throwable
	{
		this.destroy();
		super.finalize();
	}

	public String toString()
	{
		return "KerberosClientSocketFactory [KDC="
			+ this.socketToKDC.getInetAddress().getHostName()
			+ ", User=" + this.username + "]";
	}

	/**
	 * Gibt die maximal erlaubte Lnge fr zu empfangende Nachrichten der erzeugten KerberosSockets zurck.
	 */
	public int getMaxArrayLength()
	{
		return this.maxArrayLength;
	}

	/**
	 * Setzt die maximal erlaubte Lnge fr zu empfangende Nachrichten
	 * @see #getMaxArrayLength()
	 */
	public void setMaxArrayLength(int maximaleLnge) throws ZahlNichtPositivException
	{
		if(maximaleLnge < 1)
			throw new ZahlNichtPositivException("Die Maximale Array-Lnge muss grer 1 sein");
		this.maxArrayLength = maximaleLnge;
	}

	/**
	 * @return Gibt den derzeit aktuellen Logger zurck
	 */
	public Logger getLogger() 
	{
		return this.logger;
	}

	/**
	 * @param logger
	 */
	public void setLogger(Logger logger) 
	{
		if(logger == null)
			this.logger = NullLogger.get();
		else
			this.logger = logger;
	}
}

