/*
 * Created on 18.12.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.InetAddress;

import sun.net.ConnectionResetException;

import kerberos.exceptions.*;
import kerberos.net.test.KerberosTestClientSocketFactory;
import kerberos.net.test.KerberosTestServerSocket;

/**
 * Mit Hilfe dieser Factory knnen die KerberosServerSockets und die KerberosClientSocketFactory
 * erzeugt werden. Wenn man die Private Klassenmethode test auf false setzt, dann werden normale
 * KerberosServerSockets bzw. KerberosClientSocketFactorys erzeugt, wenn sie Variable auf true ist,
 * dann KerberosTestServerSockets bzw. KerberosTestClientSocketFactorys
 * @see kerberos.net.KerberosClientSocketFactory
 * @see kerberos.net.KerberosServerSocket
 * @author Stephan Koops
 */
public class KerberosFactory
{
	/**
	 * Klassenkonstante, die angibt, ob richtige Kerberos-Sockets oder nur Test-Sockets erzeugt werden sollen.
	 */
	private static final boolean test = false;
	
	/**
	 * Diese Factory-Methode erzeugt wiederum eine Factory, mit deren Hilfe 
	 * (mit der Methode <a href="KerberosClientSocketFactory.html#connect(java.lang.String,int,java.lang.String)">KerberosClientSocketFactory#connect(String, int, String)</a> )
	 * Verbindungen zu beliebig vielen Kerberos-Ziel-Servern aufgebaut werden knnen.<br>
	 * Der Konstruktor sorgt fr die Authentifizierung beim Key Distributions-Center (KDC) und speichert auch 
	 * das Ticket-Granting-Service-Ticket (TGS-Ticket).
	 * @param username Benutzername
	 * @param passwort Passwort des Benutzers. Das Passwort wird whrend der Verarbeitung zerstrt.
	 * @param kdcHostnameOrIP Netzwerkname oder IP-Adresse des KDCs
	 * @throws IOException wenn es beim Socket erzeugen oder bei der Anmeldung
	 *         beim KDC Netzwerk-Probleme gab.
	 * @throws PasswortFalschException Wenn das angegebene Passwort falsch ist.
	 * @throws KerberosException Wenn es ein Authentifizierungsproblem o.. gab.
	 * Wird auch geworfen, wenn es ein Problem beim Erzeugen des internen Paketes gab.
	 * @throws UnerwarteteNachrichtException Wird geworfen, wenn vom Server eine Nachricht 
	 * kommt, mit dem die KerberosClientSocketFactory nichts anfangen kann. Wenn der KDC
	 * korrekt arbeitet, kommt dies aber nicht vor
	 * @throws RuntimeException Wird geworfen, wenn es Problem beim Schreiben auf einen
	 * ByteArrayOutputStream gab.
	 * @see KerberosClientSocketFactoryImpl#KerberosClientSocketFactoryImpl(String, Passwort, String)
	 */
	public static KerberosClientSocketFactory clientSocketFactory(String username, Passwort passwort, String kdcHostnameOrIP)
		throws ConnectionResetException, UnerwarteteNachrichtException, NonceFalschException, PasswortDestroyedException, UnbekannterParticipalException, TicketUngueltigException, PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, ArrayZuLangException, IOException
	{
		if (test)
		{
			return new KerberosTestClientSocketFactory(username, passwort, kdcHostnameOrIP);
		} 
		return new KerberosClientSocketFactoryImpl(username, passwort, kdcHostnameOrIP);
	}
	
	/**
	 * Erzeugt einen KerberosServerSocket, auf dem neue Verbindungen erwartet werden.
	 * @see KerberosServerSocketImpl#KerberosServerSocketImpl(int, Passwort)
	 * @see java.net.ServerSocket#ServerSocket(int)
	 * @param port Port, auf dem die Verbindungen erwartet werden
	 * @param passwort Passwort dieses Dienstes
	 * @throws KerberosException Alle hier geworfenen Exceptions auer der 
	 * IOException erben von der Klasse KerberosException. Weiter 
	 * Informationen in den jeweiligen Exception-Klassen.
	 * @throws PasswortFalschException Wird geworfen, wenn das angegebene Passwort falsch ist
	 * @throws PasswortDestroyedException Wird geworfen, wenn das angegebene Passwort schn zerstrt worden ist.
	 * @throws IOException Wenn es Netzwerk-Probleme gab
	 */
	public static KerberosServerSocket serverSocket(int port, Passwort passwort)
		throws PasswortFalschException, PasswortDestroyedException, IOException
	{
		if (test)
		{
			return new KerberosTestServerSocket(port, passwort);
		} 
		return new KerberosServerSocketImpl(port, passwort);
	}

	/**
	 * Erzeugt einen KerberosServerSocket, auf dem neue Verbindungen erwartet werden.
	 * @see KerberosServerSocketImpl#KerberosServerSocketImpl(int, int, Passwort)
	 * @see java.net.ServerSocket#ServerSocket(int, int)
	 * @param port Port, auf dem dieser ServerSocket auf Verbindungen wartet
	 * @param backlog
	 * @param passwort Passwort dieses Dienstes
	 * @throws IOException Wenn es Netzwerk-Probleme gab
	 * @throws PasswortFalschException Wird geworfen, wenn das angegebene Passwort falsch ist
	 * @throws PasswortDestroyedException Wird geworfen, wenn das angegebene Passwort schn zerstrt worden ist.
	 * @throws KerberosException Alle hier geworfenen Exceptions auer der 
	 * IOException erben von der Klasse KerberosException. Weiter 
	 * Informationen in den jeweiligen Exception-Klassen.
	 */
	public static KerberosServerSocket serverSocket(int port, int backlog, Passwort passwort)
		throws PasswortFalschException, PasswortDestroyedException, IOException
	{
		if (test)
		{
			return new KerberosTestServerSocket(port, backlog, passwort);
		} 
		return new KerberosServerSocketImpl(port, backlog, passwort);
	}

	/**
	 * Erzeugt einen KerberosServerSocket, auf dem neue Verbindungen erwartet werden.
	 * @param port Port, auf dem dieser ServerSocket auf Verbindungen wartet
	 * @param backlog
	 * @param bindAddr
	 * @param passwort Passwort dieses Dienstes
	 * @throws IOException Wenn es Netzwerk-Probleme gab
	 * @throws KerberosException Alle hier geworfenen Exceptions auer die 
	 * IOException erben von der Klasse KerberosException. Weiter 
	 * Informationen in den jeweiligen Exception-Klassen.
	 * @see #serverSocket(int, int, Passwort)
	 * @see java.net.ServerSocket#ServerSocket(int, int, java.net.InetAddress)
	 * @see KerberosServerSocketImpl#KerberosServerSocketImpl(int, int, Passwort)
	 */
	public static KerberosServerSocket serverSocket(int port, int backlog, InetAddress bindAddr, Passwort passwort)
		throws PasswortFalschException, PasswortDestroyedException, IOException
	{
		if (test)
		{
			return new KerberosTestServerSocket(port, backlog, bindAddr, passwort);
		} 
		return new KerberosServerSocketImpl(port, backlog, bindAddr, passwort);
	}
}
