/*
 * Created on 07.10.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

import kerberos.exceptions.ArrayZuLangException;
import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.KerberosRuntimeException;
import kerberos.exceptions.PasswortDestroyedException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.messages.M_AP_REQ;
import kerberos.messages.M_ERROR;
import kerberos.messages.Message;
import kerberos.messages.MessageBuilder;
import kerberos.exceptions.ZahlNichtPositivException;
import kerberos.packets.PacketException;
import sun.net.ConnectionResetException;
import util.BlockingQueue;
import util.NetUtil;
import util.logger.*;

/**
 * Diese Klasse stellt die Funktionalitt eines ServerSockets zur Verfgung
 * @author Stephan Koops
 */
public class KerberosServerSocketImpl implements KerberosServerSocket
{
	/** Speichert die Maximale Array-Lnge fr den KerberosServerSocket */ 
	public static final int ARRAY_MAX_LAENGE = 256*256;
	
	/** Speichert die Standard-Maximal-Array-Lnge fr die erzeugten KerberosSockets */ 	
	public static final int STANDARD_SOCKET_ARRAY_MAX_LAENGE = 256*256;
	
	private static final boolean debug = false; 

	protected Logger logger;

	private ServerSocket serverSocket;

	private byte[] schlssel;

	/** In dieser Queue werden die fertigen, authorizierten KerberosSockets gespeichert, 
	 * wo sie zum abholen durch die Methode accept() bereitstehen */
	private BlockingQueue readySocketQueue = new BlockingQueue();

	/** In dieser Variable wird eine evtl geworfenen Exception zwischengespeichert,
	 * bis das nchste Mal die Methode accept() aufgerufen wird. */
	private Exception lastExc;

	/** Mit Hilfe dieses Objektes wird der Zugriff auf lastExc synchronisiert */
	private final Object lastExcSync = new Object();

	/** In dieser Variablen wird der Thread gespeichert, der die TCP-Verbindungen annimmt */
	private Thread acceptThread;

	/** Speichert die maximale Array-Lnge fr zu empfangene Daten der erzeugten KerberosSockets */
	private int maxArrayLength = STANDARD_SOCKET_ARRAY_MAX_LAENGE;

	/**
	 * Erzeugt einen Server-Socket, auf dem neue Verbindungen erwartet werden.
	 * @see ServerSocket#ServerSocket(int)
	 * @param port Port, auf dem die Verbindungen erwartet werden
	 * @param passwort Passwort dieses Dienstes
	 * @throws IOException Wenn es Netzwerk-Probleme gab
	 * @throws PasswortFalschException Wird geworfen, wenn das angegebene Passwort falsch ist
	 * @throws PasswortDestroyedException Wird geworfen, wenn das angegebene Passwort schn zerstrt worden ist.
	 */
	public KerberosServerSocketImpl(int port, Passwort passwort)
		throws IOException, PasswortFalschException, PasswortDestroyedException
	{
		this(new ServerSocket(port), passwort);
	}

	/**
	 * Erzeugt einen Server-Socket, auf dem neue Verbindungen erwartet werden.
	 * @see #KerberosServerSocketImpl(int, Passwort)
	 * @see java.net.ServerSocket#ServerSocket(int,int)
	 * @param port Port, auf dem dieser ServerSocket auf Verbindungen wartet
	 * @param backlog
	 * @param passwort Passwort dieses Dienstes
	 * @throws IOException Wenn es Netzwerk-Probleme gab
	 * @throws PasswortFalschException Wird geworfen, wenn das angegebene Passwort falsch ist
	 * @throws PasswortDestroyedException Wird geworfen, wenn das angegebene Passwort schn zerstrt worden ist.
	 */
	public KerberosServerSocketImpl(int port, int backlog, Passwort passwort)
		throws IOException, PasswortFalschException, PasswortDestroyedException
	{
		this(new ServerSocket(port, backlog), passwort);
	}

	/**
	 * @see #KerberosServerSocketImpl(int, Passwort)
	 * @see #KerberosServerSocketImpl(int, int, Passwort)
	 * @see java.net.ServerSocket#ServerSocket(int,int,java.net.InetAddress)
	 * @param port Port, auf dem dieser ServerSocket auf Verbindungen wartet
	 * @param backlog
	 * @param bindAddr
	 * @param passwort Passwort dieses Dienstes
	 * @throws IOException Wenn es Netzwerk-Probleme gab
	 * @throws kerberos.exceptions.KerberosException Alle hier geworfenen Exceptions 
	 * (auer IOException) sind von der KerberosException abgeleitet. Einzelheiten siehe
	 * in den jeweiligen Exceptions.
	 */
	public KerberosServerSocketImpl(
		int port,
		int backlog,
		InetAddress bindAddr,
		Passwort passwort)
		throws IOException, PasswortFalschException, PasswortDestroyedException
	{
		this(new ServerSocket(port, backlog, bindAddr), passwort);
	}

	/**
	 * Erzeugt einen Server-Socket, auf dem neue Verbindungen erwartet werden.
	 * @param serverSocket vorhandener ServerSocket
	 * @param passwort Passwort, aus dem der Schlssel des Servers abgeleitet wird. Das Passwort
	 *        wird beim Auslesen des Schlssels zerstrt. 
	 * @throws PasswortFalschException Wird geworfen, wenn das angegebene Passwort falsch ist
	 * @throws PasswortDestroyedException Wird geworfen, wenn das angegebene Passwort schn zerstrt worden ist.
	 */
	protected KerberosServerSocketImpl(ServerSocket serverSocket, Passwort passwort)
		throws PasswortFalschException, PasswortDestroyedException
	{
		this.serverSocket = serverSocket;
		this.schlssel = passwort.getSchlssel();
		this.startAccAndAuth();
	}
	
	private void startAccAndAuth()
	{
		/**
		 * Dieser Thread luft ist dafr zustndig,
		 * dass im Hintergrund immer Verbindungen angenommen werden und die
		 * Authentifizierung stattfindet. Die erfolgreich angenommenen und
		 * authentifizierten Verbindungen werden als KerberosSocket in die
		 * Queue readySocketQueue aufgenommen und stehen dann dort fr die
		 * Methode accept bereit. Nicht authentifizierte Clients werden ignoriert.
		 * @see readySocketQueue
		 * @author Stephan
		 */
		acceptThread = new Thread()
		{
			public void run()
			{
				while (! this.isInterrupted())
				{
					try
					{
						Socket tcpSocket = serverSocket.accept();
						if(this.isInterrupted())
						{
							tcpSocket.close();
							serverSocket.close();
						} 
						else
						{
							new AuthThread(tcpSocket).start();
						} 
					}
					catch (IOException ioe)
					{
						this.interrupt(); // Nach einer IOException wird der Thread beendet
						if(serverSocket.isClosed())
						{
							addExcWithoutPrint(ioe);
						}
						else
						{
							addExc(ioe);
						}
					}
				}
			}
		};
		acceptThread.start();
	}
	
	/**
	 * Dieser Thread und ist dafr zustndig,
	 * dass fr eine angekommene Verbindungen die
	 * Authentifizierung stattfindet. Die erfolgreich angenommenen und
	 * authentifizierten Verbindungen werden als KerberosSocket in die
	 * Queue readySocketQueue aufgenommen und stehen dann dort fr die
	 * Methode accept bereit
	 * @see #readySocketQueue
	 * @author Stephan
	 */
	class AuthThread extends Thread
	{
		private Socket tcpSocket;
		AuthThread(Socket tcpSocket)
		{
			this.tcpSocket = tcpSocket;
		}
		public void run()
		{
			try
			{
				KerberosSocket kerbSocket = authentifizierung(tcpSocket);
				if (kerbSocket != null)
				{
					readySocketQueue.add(kerbSocket);
				}
			}
			catch (Exception exc)
			{
				addExc(exc);
			}
		}
	}

	/**
	 * Diese Methode speichert eine geworfene Exception in der dazugehrigen Struktur,
	 * wie auch immer die gerade aussieht.
	 * @param exc geworfene Exception, die spter mal ausgegeben werden soll.
	 */
	protected void addExc(Exception exc)
	{
		// Zur Zeit wird genau die letzte geworfene Exception gespeichert,
		// bis sie in der Methode accept() geworfen wird. Dann wird sie gelscht
		addExcWithoutPrint(exc); 
		logger.log(exc, 10);
	}

	protected void addExcWithoutPrint(Exception exc)
	{
		lastExc = exc;
	}

	/**
	 * Diese Methode ist fr die Authentifizierung des anfragenden Clients zustndig.
	 * @param socket TCP-Socket, auf dem die Anfrage kam
	 * @return den IKerberosSocket, wenn alles OK war oder null, wenn sich der Client
	 * nicht authentifizieren konnte
	 */
	private KerberosSocket authentifizierung(Socket socket) throws ConnectionResetException, ArrayZuLangException, IOException, PacketException
	{
		String methode = "KerberosServerSocket.authentifizierung()"; 
		byte[] cryptDaten = NetUtil.readByteArray(socket, this.maxArrayLength); // Anmeldung des Clients auslesen
		Message message;
		try
		{
			message = MessageBuilder.currentBuilder().readMessage(cryptDaten, this.schlssel);
		}
		catch (PasswortFalschException e)
		{
			NetUtil.sendError(socket, M_ERROR.M_AP_REQ, M_ERROR.ERR_BADMATCH);
			return null;
		}
		catch (UngltigerMessageTypException e)
		{
			NetUtil.sendError(socket, M_ERROR.M_AP_REQ, M_ERROR.ERR_MSG_TYPE);
			return null;
		}
		catch (ClientnameFalschException cfe)
		{
			logger.log(methode+": ClientnameFalschException " + cfe.getMessage(), 10);
			return null;
		}
		if (message instanceof M_AP_REQ)
		{
			M_AP_REQ posAntw = (M_AP_REQ)message;
			Message.checkZeit(posAntw);
			//logger.log(methode+": Die Verbindung wurde angenommen");
			return acceptOK(posAntw, socket);
		}
		// Wenn der Nachrichtentyp nicht zu erwarten ist, dann wird auf die nchste Nachricht gewartet
		logger.log("Unerwartete Nachricht im KerberosServerSocket: Typ="+message.getClass().getName(), 10);
		return null;
	}

	/**
	 * Nimmt eine Verbindung von einem Client an und gibt einen KerberosSocket zurck.
	 * Da das Annehmen der TCP-Verbindungen und danach die Authentifizierung in einem anderen Thread luft,
	 * wird die zuletzt aufgetretende Exceptions gesammelt und beim Aufruf der Methode accept geworfen. 
	 * @see java.net.ServerSocket#accept()
	 * @throws IOEXception Wenn es ein Netzwerkproblem gab.
	 * @throws KerberosRuntimeException, wenn es beim Schreiben auf einen Memory-Stream eine IO-Exception gab.
	 * Dies sollte aber eigentlich nicht vorkommen. Die IOException wird dann als Grund der RuntimeException angegeben.
	 * Eine RuntimeException kann auch geworfen werden, wenn festgestellt wurde, dass das nicht autorisierte 
	 * Nachrichten empfangen wurden. Dies passiert dann, wenn jemand ungltige Daten schickt.
	 */
	public KerberosSocket accept() throws IOException, KerberosRuntimeException
	{
		synchronized (lastExcSync)
		{
			if (lastExc != null)
			{
				Exception exc = lastExc;
				lastExc = null;
				if (exc instanceof IOException)
					throw (IOException)exc;
				if (exc instanceof RuntimeException)
					throw (RuntimeException)exc;
			}
		}
		if(!serverSocket.isClosed())
			return (KerberosSocket) this.readySocketQueue.get();
		throw new IOException("Der KerberosServerSocket wurde geschlossen");
	}

	/**
	 * Gibt eine positive Antwort an den Client zurck und erzeugt den KerberosSocket 
	 * @param posAntw
	 * @param socket TCP-Socket mit der angenommenen Verbindung
	 * @return den erzeugten KerberosSocket
	 * @throws IOException wenn es einen TCP-Fehler gab.
	 */
	private KerberosSocket acceptOK(M_AP_REQ posAntw, Socket socket) throws IOException
	{
		String methode = "KerberosServerSocket.acceptOK(..)";
		long nonce = posAntw.getNonce();
		String user = new String(posAntw.getUsername());
		byte[] schlssel_CS = posAntw.getSchluessel_C_S();
		byte[] loginOK;
		if (debug)
			logger.log(methode+" Antwort an Client mit Schlssel{C,S}="+Passwort.toZahl(schlssel_CS), 5);
		try
		{
			loginOK = MessageBuilder.currentBuilder().buildData(new byte[0], schlssel_CS, nonce);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
		NetUtil.writeArray(socket, loginOK);
		return new KerberosSocketS(socket, schlssel_CS, user, this.maxArrayLength, this.logger);
	}

	/**
	 * @see java.net.ServerSocket#close()
	 */
	public void close() throws IOException
	{
		this.closeLigth();
		readySocketQueue.clear();
	}

	/**
	 * @see kerberos.net.KerberosServerSocket#closeLigth()
	 */
	public void closeLigth() throws IOException
	{
		this.serverSocket.close();
	}

	// Goodies

	/**
	 * Gibt die lokale Adresse dieses ServerSockets zurck
	 * @see java.net.Socket#getInetAddress()
	 */
	public InetAddress getInetAddress()
	{
		return this.serverSocket.getInetAddress();
	}


	public int getLocalPort()
	{
		return this.serverSocket.getLocalPort();
	}

	/**
	 * @return Gibt true zurck, wenn der ServerSocket ganz geschlossen ist,
	 * d.h. wenn keine Verbindungen mehr angenommen werden und auch keine schon 
	 * authentifizierten Verbindungen mehr auf ihr accept() warten. 
	 * @see java.net.ServerSocket#isClosed()
	 * @see #isClosedLight()
	 * @see #close()
	 */
	public boolean isClosed()
	{
		return this.isClosedLight() && this.readySocketQueue.isEmpty();
	}

	/**
	 * @return Gibt true zurck, wenn der ServerSocket keine neuen Verbindungen mehr annimmt.
	 * Ob noch schon authorizierte Verbindungen auf ihr accept warten, wird nicht geprft.
	 * @see #isClosed()
	 * @see #closeLigth() 
	 */
	public boolean isClosedLight()
	{
		return this.serverSocket.isClosed();
	}

	public String toString()
	{
		return "Kerberos" + this.serverSocket.toString();
	}

	/**
	 * Gibt die maximal erlaubte Lnge fr zu empfangende Arrays der 
	 * erzeugten KerberosSockets zurck.
	 */
	public int getMaxArrayLength()
	{
		return this.maxArrayLength;
	}

	/**
	 * Setzt die maximal erlaubte Lnge fr zu empfangende Arrays der 
	 * erzeugten KerberosSockets zurck.
	 * @see #getMaxArrayLength()
	 */
	public void setMaxArrayLength(int i) throws ZahlNichtPositivException
	{
		this.maxArrayLength = i;
	}
	/**
	 * @return Gibt den derzeit aktuellen Logger zurck
	 */
	public Logger getLogger() 
	{
		return this.logger;
	}

	/**
	 * @param logger
	 */
	public void setLogger(Logger logger) 
	{
		if(logger == null)
			this.logger = NullLogger.get();
		else
			this.logger = logger;
	}
}
