/*
 * Created on 07.10.2003
 */
package kerberos.net;

import java.io.IOException;
import java.net.InetAddress;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.ErstEmpfangenException;
import kerberos.exceptions.ErstSendenException;
import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.SendeArrayZuLangException;
import kerberos.exceptions.TicketUngueltigException;
import kerberos.exceptions.UnerwarteteNachrichtException;
import kerberos.exceptions.ZahlNichtPositivException;

/**
 * <h2>Die Kerberos-Socket-API</h2>
 * <p>Programmiert wird gegen die Interfaces <a href="KerberosServerSocket.html">KerberosServerSocket</a>,
 * <a href="KerberosSocketFactory.html">KerberosSocketFactory</a> und 
 * <a href="KerberosSocket.html">KerberosSocket</a>.<br>
 * Das Interface <a href="KerberosSocket.html">KerberosSocket</a> funktioniert im Prinzip wie ein normaler Socket, 
 * nur eben verschlsselt und authentifiziert ber Kerberos. Erzeugt wird ein KerberosSocket
 * auf der Client-Seite von einer 
 * <a href="KerberosClientSocketFactory.html">KerberosClientSocketFactory</a>, mit der Methode
 * <a href="KerberosClientSocketFactory.html#connect(java.lang.String, int, java.lang.String)">connect(String, int String)</a>.
 * Die KerberosClientSocketFactory sorgt fr die Authentifizierung gegenber dem KDC.
 * Im <a href="KerberosSocket.html">KerberosSocket</a> werden allerdings keine Streams zur Verfgung gestellt, 
 * sondern die entsprechenden print- und read-Methoden sind direkt im Socket enthalten. Durchs Netz werden immer
 * Byte-Arrays bertragen. Wenn die Methode print(String) aufgerufen wird, dann werden die einzelnen Bytes des Strings
 * durchs Netz geschickt. Es ist auf der Empfangsseite egal, ob eine Nachricht als Sring oder Byte-Array gelesen 
 * wird.<br>
 * Auf der Serverseite arbeitet der KerberosServerSocket wie ein normaler java.net.ServerSocket.</p>
 * <p>Als konkrete Implementationen stehen die Klassen <a href="KerberosServerSocketImpl.html">KerberosServerSocketImpl</a>,
 * <a href="KerberosClientSocketFactoryImpl.html">KerberosClientSocketFactoryImpl</a> zu Verfgung, welche
 * die KerberosSockets erzeugen. Zum Erzeugung von KerberosClientsSocketFactory und KerberosServerSocket steht
 * die <a href="KerberosFactory.html">KerberosFactory</a> zur Verfgung.</p>
 * <p>
 * Damit die LDAP-Leute schon praktisch gegen die Interfaces arbeiten knnen, stehen Testklassen zur Verfgung.
 * </p><p>
 * Dieses Interfaces dient fr eine konkrete Verbindung zu einem Client. Diese ist dann schon aufgebaut
 * und steht erfolgreich.</p>
 * @see kerberos.net.KerberosFactory
 * @see kerberos.net.KerberosServerSocket
 * @see kerberos.net.KerberosServerSocketImpl
 * @see kerberos.net.KerberosSocketC
 * @see kerberos.net.KerberosSocketS
 * @see kerberos.net.KerberosClientSocketFactory
 * @see kerberos.net.KerberosClientSocketFactoryImpl
 * 
 * @author Stephan Koops
 */
public interface KerberosSocket
{
	/**
	 * Schickt ein Byte-Array (oder einen String) sofort verschlsselt durchs Netz. 
	 * Damit die Kerberos-Kontrolle, ob niemand alte Nachrichten nochmal sendet (replay), funktioniert, muss
	 * immer abwechslend eine Nachricht gesendet und eine empfangen werden, wobei der Client anfangen 
	 * muss zu senden. Wenn die Anwendung keine Daten zu versenden hat, dann kann sie null, ein Byte-Array 
	 * der Lnge 0 oder einen leeren String (Lnge 0) versenden. null kommt auf der gegenberliegenden Seite
	 * nicht als null an, sondern als Byte-Array mit der Lnge 0 bzw. als String der Lnge 0.
	 * Alle Exceptions auer der IOException und der ConnectionResetException sind von KerberosException abgeleitet, 
	 * so dass es reicht, die IOException und die KerberosException abzufangen, wenn man die Exceptions nicht weiter 
	 * auswerten mchte. 
	 * @param zeichen byte-Array (oder String), das/der verschickt werden soll.
	 * @throws IOException Wenn es Netzwerk-Probleme gab.
	 * @throws PasswortFalschException Wenn eine Nachricht nicht entschlsselt werden konnte, weil jemand ungltige
	 * Nachrichten eingespeit hat
	 * @throws KerberosException Alle hier geworfenen Exceptions auer die IOException erben von der Klasse
	 * KerberosException. Weiter Informationen in den jeweiligen Exception-Klassen.
	 * @see #read()
	 */
	public void print(byte[] zeichen)
		throws IOException, PasswortFalschException, ClientnameFalschException, 
			   UnerwarteteNachrichtException, NonceFalschException, ErstEmpfangenException; // die Erst...Exc stimmt jetzt !!!

	/**
	 * Schickt einen String durchs Netz
	 * @see #print(byte[])
	 * @see #readString()
	 */
	public void print(String zeichen)
		throws IOException, PasswortFalschException, ClientnameFalschException, 
			   UnerwarteteNachrichtException, NonceFalschException, ErstEmpfangenException; // die Erst...Exc stimmt jetzt !!!

	/**
	 * <P>Jetzt die synchrone Variante ...<br>
	 * Sendet das bergebenen Byte-Array (bzw. den String) verschlsselt ab und wartet 
	 * auf die Antwort vom Zieldienst.
	 * Liest die nchsten Daten aus und gibt sie als Byte-Array (bzw. als String) zurck. Es ist egal, 
	 * ob die Daten von der gegenberliegenden Seite als String oder als Byte-Array gesendet wurden.<br>
	 * Die Methode blockiert, wenn nicht genug Daten von der Gegenseite zur Verfgung stehen.</p>
	 * Alle Exceptions auer der IOException sind von KerberosException abgeleitet, so dass es reicht, die
	 * IOException und die KerberosException abzufangen, wenn man die Exceptions nicht weiter auswerten mchte. 
	 * @param zeichen
	 * @return Schickt ein byte-Array durchs Netz, wartet auf die Antwort des Gegenber und gint das empfangene Byte-Array zurck
	 * @throws IOException
	 * @throws KerberosException Alle hier geworfenen Exceptions auer IOException und ConnectionResetException 
	 * erben von der Klasse KerberosException. Weiter Informationen in den jeweiligen Exception-Klassen.
	 * @throws SendeArrayZuLangException Wird geworfen, wenn das gesendete Array zu lang war.
	 * @see #print(byte[])
	 * @see #print(String)
	 * @see #read()
	 * @see #readString()
	 */
	public byte[] printRead(byte[] zeichen)
		throws SendeArrayZuLangException, IOException, PasswortFalschException, ClientnameFalschException, TicketUngueltigException, 
			   UnerwarteteNachrichtException, NonceFalschException, ErstEmpfangenException; // die Erst...Exc stimmt jetzt !!!
			   
	/**
	 * @see #printRead(byte[])
	 */
	public String printRead(String zeichen)
		throws SendeArrayZuLangException, IOException, PasswortFalschException, ClientnameFalschException, TicketUngueltigException, 
			   UnerwarteteNachrichtException, NonceFalschException, ErstEmpfangenException; // die Erst...Exc stimmt jetzt !!!

	/**
	 * Liest die nchsten Daten aus und gibt sie als Byte-Array bzw. als String zurck. Es ist egal, ob
	 * die Daten von der anderen Seite als Byte-Array oder als String gesendet wurden.
	 * Blockiert, wenn nicht genug Daten zur Verfgung stehen.
	 * Alle Exceptions auer der IOException sind von KerberosException abgeleitet, so dass es reicht, die
	 * IOException und die KerberosException abzufangen, wenn man die Exceptions nicht weiter auswerten mchte. 
	 * @return Gibt das nchste zur Verfgung stehende Byte-Array zurck. Blockiert, 
	 * wenn keine (oder nicht genug Daten) zur Verfgung stehen.
	 * @throws IOException Wenn es Netzwerkprobleme gab
	 * @throws KerberosException Alle hier geworfenen Exceptions auer die IOException erben von der Klasse
	 * KerberosException. Weiter Informationen in den jeweiligen Exception-Klassen.
	 * @throws SendeArrayZuLangException Wird geworfen, wenn das letzte gesendete Array zu lang war. Der Entwickeler 
	 * muss berlegen, ob er programmiert bekommt, dass er die Nachricht nochmal sendet oder die Verbindung beendet.
	 * In der Methode printRead(..) ist dies sicher einfacher.
	 * @see #printRead(byte[])
	 * @see kerberos.exceptions.NonceFalschException
	 * @see kerberos.exceptions.PasswortFalschException
	 * @see kerberos.exceptions.ClientnameFalschException
	 * @see kerberos.exceptions.UnerwarteteNachrichtException
	 * @see kerberos.exceptions.ErstEmpfangenException
	 */
	public byte[] read()	
		throws SendeArrayZuLangException, IOException, PasswortFalschException, ClientnameFalschException, NonceFalschException, 
			   UnerwarteteNachrichtException, TicketUngueltigException, ErstSendenException;  // die Erst...Exc stimmt jetzt !!!
	
	/**
	 * Liest einen String aus dem Netz.
	 * @see #read()
	 */
	public String readString()	
		throws SendeArrayZuLangException, IOException, PasswortFalschException, ClientnameFalschException, NonceFalschException, UnerwarteteNachrichtException, ErstSendenException, TicketUngueltigException,
			   ErstSendenException;  // die Erst...Exc stimmt jetzt !!!
	
	/**
	 * Beendet die Verbindung.
	 * @see java.net.Socket#close()
	 * @throws IOException
	 */
	public void close() throws IOException;
	
	/**
	 * @return Gibt zurck, ob der Socket geschlossen ist.
	 */
	public boolean isClosed();
	
	/**
	 * Diese Methode gibt den Namen des Benutzers zurck, der sich eingeloggt hat. Dies gilt auch 
	 * auf der Client-Seite.
	 */
	public String getUsername();

	/**
	 * Gibt die maximal erlaubte Lnge fr zu empfangende Arrays zurck.
	 */
	public int getMaxArrayLength();

	/**
	 * Setzt die maximal erlaubte Lnge fr zu empfangende Nachrichten
	 * @see #getMaxArrayLength()
	 */
	public void setMaxArrayLength(int i) throws ZahlNichtPositivException;

	/**
	 * Gibt die IP-Adresse zurck, zu dem der Socket verbunden ist. 
	 * @see java.net.Socket#getInetAddress()
	 */
	public InetAddress getInetAddress();
	
	/**
	 * Gibt den port zurck, zu dem der Socket verbunden ist.
	 * @see java.net.Socket#getPort()
	 */
	public int getPort();
}
