/*
 * Created on 07.10.2003
 */
package kerberos.net;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;

import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.ErstEmpfangenException;
import kerberos.exceptions.ErstSendenException;
import kerberos.exceptions.KerberosRuntimeException;
import kerberos.exceptions.NonceFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.SendeArrayZuLangException;
import kerberos.exceptions.TicketUngueltigException;
import kerberos.exceptions.UnerwarteteNachrichtException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.messages.M_AP;
import kerberos.messages.M_ERROR;
import kerberos.messages.Message;
import kerberos.messages.MessageBuilder;
import kerberos.exceptions.ZahlNichtPositivException;
import kerberos.packets.PacketException;
import sun.net.ConnectionResetException;
import util.NetUtil;
import util.logger.*;

/**
 * Diese Klasse dient fr eine konkrete Verbindung zu einem Client. Nach dem Ende des Konstruktors
 * ist die Verbindung aufgebaut und steht erfolgreich, gegeneinander authentifiziert und verschlsselt.<br>
 * Um die Authentifizierung etc kmmert sich die KerberosClientSocketFactory bzw. der KerberosServerSocket.
 * Nach dem Aufbauen der Verbindung muss der Client erst was senden, und dann muss immer abwechslend gesendet
 * und empfangen werden !!!!!!! Sonst wird eine RuntimeException geworfen (Der Grund ist dann eine
 * ErstSendenException bzw. eine erstEmpfangenException)
 * @author Stephan Koops
 */
abstract class KerberosSocketAbstr implements KerberosSocket
{
	private static final boolean debug = false;
	
	protected Logger logger;
	
	private Socket socket;
	private OutputStream outputStream;
	private InputStream inputStream;
	protected String username;

	/**
	 * In dieser Methode wird der Schlssel gespeichert, mit dem diese 
	 * Verbindung verschlsselt wird. ( K{C,S} )
	 */
	protected byte[] schlssel_CS;

	/** Speichert die maximale Message-Gre (Array-Lnge fr zu empfangene Daten */
	private int maxArrayLength;
	
	/**
	 * Neues KerberosSocket-Objekt erzeugen. Die Authentifizierung etc. wird vorher von der erzeugenden Instanz
	 * (also der KerberosClientSocketFactory bzw. dem KerberosServerSocket) erledigt. 
	 * @param schlssel Schlssel, mit dem die Verbindung verschlsselt wird, also Schlssel K_{C,S}
	 * @param username Name des sich einloggenden Benutzers.
	 * @param username Benutzername des sich einloggenden Benutzers.
	 * @param maxArrayLength Maximale Array-Lnge der Daten, die die erzeugten KerberosSockets haben drfen
	 * @throws IOException Wenn es Netzwerkprobleme gab
	 */
	KerberosSocketAbstr(Socket socket, byte[] schlssel_CS, String username, int maxArrayLength, Logger logger) throws IOException
	{
		this.socket = socket;
		this.outputStream = socket.getOutputStream();
		this.inputStream = socket.getInputStream();
		this.schlssel_CS = schlssel_CS;
		this.username = username;
		this.maxArrayLength = maxArrayLength;
		this.logger = logger;
	}

	/**
	 * @see #print(byte[])
	 * @see kerberos.net.KerberosSocket#print(java.lang.String)
	 */
	public void print(String string)
		throws IOException, KerberosRuntimeException, ErstEmpfangenException
	{
		if (string == null)
			print(new byte[0]);
		else
			print(string.getBytes());
	}

	/**
	 * Schickt ein Byte-Array durchs Netz.
	 * @param zeichen
	 * @throws IOException
	 * @throws RuntimeException, wenn es beim Schreiben auf einen Memory-Stream eine IO-Exception gab.
	 * Dies sollte aber eigentlich nicht vorkommen. Die IOException wird als Grund der RuntimeException angegeben.  
	 */
	public void print(byte[] zeichen) throws IOException, KerberosRuntimeException, ErstEmpfangenException
	{
		byte[] daten;
		if (zeichen == null)
			zeichen = new byte[0];
		try
		{
			daten = buildCryptData(zeichen);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
		NetUtil.writeArray(socket, daten);
	}

	/**
	 * Erzeugt das zu versendennde byte[] mit den verschlsselten Daten 
	 * @throws RuntimeException, wenn es beim Schreiben auf einen Memory-Stream eine 
	 * IO-Exception gab. Dies sollte aber eigentlich nicht vorkommen. Die IOException 
	 * wird als Grund der RuntimeException angegeben.  
	 */
	protected abstract byte[] buildCryptData(byte[] clearData) throws PacketException, ErstEmpfangenException;

	/**
	 * Versendet eine String mit Kerberos verschlsselt durchs Netz, wartet auf den Antwort-String und gibt diesen zurck
	 * @see kerberos.net.KerberosSocket#printRead(java.lang.String)
	 */
	public String printRead(String string)
		throws
			ConnectionResetException, ErstEmpfangenException, PasswortFalschException, ClientnameFalschException, NonceFalschException, UnerwarteteNachrichtException, TicketUngueltigException, SendeArrayZuLangException, IOException
	{
		return new String(printRead(string.getBytes()));
	}

	/**
	 * Jetzt die synchrone Variante ...<br>
	 * Sendet das bergebene byte[] verschlsselt ab und gibt ne Antwort zurck
	 * @see KerberosSocket#printRead(byte[])
	 * @see #print(byte[])
	 * @see #read()
	 * @param string
	 * @return das nchste, zur Verfgung stehende byte[] zurck
	 * @throws IOException bei Netzwerkproblemen
	 */
	public byte[] printRead(byte[] zeichen)
		throws
			ErstEmpfangenException, ConnectionResetException, PasswortFalschException, ClientnameFalschException, NonceFalschException, UnerwarteteNachrichtException, TicketUngueltigException, SendeArrayZuLangException, IOException
	{
		this.print(zeichen);
		try
		{
			return this.read();
		}
		catch (ErstSendenException e)
		{
			// kann hier nicht auftreten, da gerade gesendet wurde
			throw new KerberosRuntimeException(e);
		}
	}

	/**
	 * Gibt die nchste Nachricht als String zurck
	 * @see #read()
	 * @see kerberos.net.KerberosSocket#readString()
	 */
	public String readString()
		throws ConnectionResetException, PasswortFalschException, ClientnameFalschException, NonceFalschException, UnerwarteteNachrichtException, ErstSendenException, TicketUngueltigException, SendeArrayZuLangException, IOException
	{
		return new String(read());
	}

	/**
	 * Gibt das nchste byte[] zurck oder null, wenn die empfangene Nachricht keine M_AP-Nachricht ist
	 * @see M_AP
	 * @see kerberos.net.KerberosSocket#read()
	 */
	public byte[] read()
		throws
			PasswortFalschException,
			ClientnameFalschException,
			NonceFalschException,
			UnerwarteteNachrichtException,
			SendeArrayZuLangException, ErstSendenException, TicketUngueltigException, ConnectionResetException, IOException
	{
		StringBuffer m = new StringBuffer("KerberosSocket");
		if (this.getClass().getName().indexOf("SocketC") >= 0)
			m.append("[C]");
		else if (this.getClass().getName().indexOf("SocketS") >= 0)
			m.append("[S]");
		m.append(".read()");
		String methode = m.toString();
		checkReadAllowed();
		byte[] cryptDaten = NetUtil.readByteArray(this.socket, this.maxArrayLength + zustzlichePaketgreDurchPacketBuilder());
		Message message;
		try
		{
			if(debug)
				System.out.println(methode + " entschlsseln mit Schlssel{C,S} " + Passwort.toZahl(this.schlssel_CS));
			message = MessageBuilder.currentBuilder().readMessage(cryptDaten, this.schlssel_CS);
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		}
		catch (UngltigerMessageTypException umte) // sollte eigentlich nicht vorkommen
		{
			throw KerberosRuntimeException.throwWith(umte);
		}
		if (message instanceof M_AP)
		{
			M_AP aw = (M_AP)message;
			handleNonce(aw);
			if(debug)
			{
				System.out.println("KerberosSocket.read() M_AP-Typ = " + message.getClass().getName());
				System.out.println("KerberosSocket.read() Daten =" + Passwort.toZahl(aw.getDaten()) + " = " + new String(aw.getDaten()));
			}
			return aw.getDaten();
		}
		if (message instanceof M_ERROR)
		{
			M_ERROR errMessage = (M_ERROR)message;
			errMessage.checkForReasonInvalidTicket();
			errMessage.checkForReasonArrayZuLang();
			throw new UnerwarteteNachrichtException(errMessage);
		}
		throw new UnerwarteteNachrichtException(message);
	}

	/**
	 * Gibt den Benutzernamen des eingeloggten Users zurck
	 */
	public String getUsername()
	{
		return this.username;
	}

	public String toString()
	{
		String sts = this.socket.toString();
		return "Kerberos" + sts.substring(0, sts.length() - 2) + " User=" + this.getUsername() + "]";
	}

	/**
	 * Wird nach dem Empfangen einer Nachricht aufgerufen.
	 * Testet auf der Client-Seite, ob der Nonce-Wert der empfangenen Nachricht stimmt.
	 * Auf der Serverseite wird der Nonce-Wert gespeichert, um ihn gleich wieder zurck schicken zu knnen  
	 * @param message
	 * @throws NonceFalschException Wird geworfen, wenn der Nonce-Wert der Nachricht ungltig ist
	 * @throws ErstSendenException kann nur auf der ServerSeite geworfen werden
	 * @see #read()
	 */
	protected abstract void handleNonce(M_AP message) throws NonceFalschException, ErstSendenException;

	/**
	 * Testet, ob das Empfangen einer Nachricht erlaubt ist
	 * @throws ErstSendenException Wird geworfen, wenn z.Zt. keine Nachricht gelesen werden darf.
	 */
	protected abstract void checkReadAllowed() throws ErstSendenException;

	/**
	 * Testet, ob das Senden einer Nachricht erlaubt ist
	 * @throws ErstEmpfangenException Wird geworfen, wenn z.Zt. keine Nachricht gesendet werden darf.
	 */
	protected abstract void checkPrintAllowed() throws ErstEmpfangenException;

	/**
	 * @see kerberos.net.KerberosSocket#close()
	 */
	public void close() throws IOException
	{
		outputStream.close();
		inputStream.close();
		socket.close();
	}

	/**
	 * @see kerberos.net.KerberosSocket#isClosed()
	 */
	public boolean isClosed()
	{
		return socket.isClosed();
	}

	/**
	 * @see kerberos.net.KerberosSocket#getInetAddress()
	 */
	public InetAddress getInetAddress()
	{
		return this.socket.getInetAddress();
	}

	/**
	 * @see kerberos.net.KerberosSocket#getPort()
	 */
	public int getPort()
	{
		return this.socket.getPort();
	}

	/**
	 * @see KerberosSocket#getMaxArrayLength()
	 */
	public int getMaxArrayLength()
	{
		return this.maxArrayLength;
	}

	/**
	 * @see KerberosSocket#setMaxArrayLength(int)
	 */
	public void setMaxArrayLength(int maximaleLnge) throws ZahlNichtPositivException
	{
		if(maximaleLnge < 1)
			throw new ZahlNichtPositivException("Die Maximale Array-Lnge muss grer 1 sein");
		this.maxArrayLength = maximaleLnge;
	}

	/**
	 * Diese Methode soll die zustzliche Gre durch den PacketBuilder zurckgeben 
	 */	
	abstract int zustzlichePaketgreDurchPacketBuilder();
	
	/**
	 * Wenn null bergeben wird, dann wird nicht mitgeloggt
	 * @param logger
	 */
	public void setLogger(Logger logger)
	{
		if(logger == null)
			this.logger = NullLogger.get();
		else
			this.logger = logger;
	}
}
