/*
 * Created on 13.10.2003
 */
package kerberos.net;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import kerberos.exceptions.PasswortDestroyedException;

/**
 * Diese Klasse kapselt ein Passwort. 
 * Passworte sollten nicht als String gespeichert werden, da diese read-only sind, d.h.
 * man kann die enthaltenen Zeichen nicht lschen, so dass es sein kann, dass die Zeichen
 * unkontrolliert im Speicher stehen und sie irgendjemand lesen kann. In dieser Klasse wird
 * im Konstruktor nur der Schlssel aus den bergebenen Werten erzeugt und gespeichert.
 * Das bergebene Byte-Array wird nach dem Erzeugen des Passwort-Objektes sofort zerstrt.
 * @author Stephan Koops
 * @see #destroy()
 */
public class Passwort
{
	private boolean destroyed = false;
	private byte[] schlssel;

	/**
	 * Erzeugt ein neues Passwort-Objekt. In diesem Objekt wird die Zeichenkette aber gar nicht gespeichert.
	 * Aus dem bergebenen Char-Array bzw. dem bergebenen Byte-Array wird der Schlssel gleich ausgelesen 
	 * und gespeichert. Das bergebene Array wird dann zerstrt.
	 * @see #destroy()
	 * @param passwort
	 */
	public Passwort(byte[] passwort)
	{
		try
		{
			MessageDigest md = MessageDigest.getInstance("MD5");
			this.schlssel = md.digest(passwort); // Berechnet einen Krypto-Hash
		}
		catch (NoSuchAlgorithmException e)
		{
			throw new RuntimeException(e);
		}
		finally
		{
			destroyArray(passwort);
		}
	}

	/**
	 * @see #Passwort(byte[])
	 * @param passwort
	 */
	public Passwort(char[] passwort)
	{
		this(toByteArray(passwort));
		destroyArray(passwort);
	}

	/**
	 * Erzeugt ein Passwort-Objekt mit den Zeichen, die durch den String reprsentiert werden.
	 * Dieser Aufruf sollte nur fr Testzwecke verwendet werden, da die Zeichen in den Strings 
	 * nicht explizit gelscht werden knnen. 
	 * @param passwort
	 * @see #Passwort(byte[])
	 */
	public Passwort(String passwort)
	{
		this(passwort.getBytes());
	}

	/**
	 * Wandelt ein char[] in ein byte[] um
	 * @param passwort
	 */
	public static byte[] toByteArray(char[] passwort)
	{
		byte[] bytes = new byte[passwort.length];
		for(int i=0; i<passwort.length; i++)
		{
			bytes[i] = (byte)passwort[i];
		}
		return bytes;
	}

	public synchronized void destroy()
	{
		this.destroyed = true;
		destroyArray(this.schlssel);
	}

	/**
	 * berschreibt alle Positionen des bergebenen Arrays mit 0, so dass die Werte
	 * des Arrays nicht mehr ausgelesen werden knnen.
	 * @see #destroy()
	 */
	public static void destroyArray(byte[] array)
	{
		for(int i=0; i<array.length; i++)
		{
			array[i] = 0;
		}
	}

	/**
	 * berschreibt alle Positionen des bergebenen Arrays mit 0
	 * @see #destroy()
	 */
	public static void destroyArray(char[] array)
	{
		for(int i=0; i<array.length; i++)
		{
			array[i] = 0;
		}
	}

	/**
	 * Gibt den MD5-Schlssel fr dieses Passwort zurck.
	 * @throws RuntimeException Wird geworfen, wenn der Verschlsselungsalgorithmus MD5 nicht zur Verfgung steht.
	 */
	public synchronized byte[] getSchlssel() throws PasswortDestroyedException
	{
		if(this.destroyed)
			throw new PasswortDestroyedException();
		return this.schlssel;
	}

	/**
	 * Gibt die einzelnen Zahlen des Arrays durch Kommas getrennt als String zurck.
	 * @param array
	 */
	public static String toZahl(byte[] array)
	{
		if(array.length == 0)
			return "";
		StringBuffer stb = new StringBuffer(array.length*4);
		append(stb, array[0]);
		for(int i=1; i<array.length; i++)
		{
			stb.append(",");
			append(stb, array[i]);
		}
		return stb.toString();
	}
	
	private static void append(StringBuffer stb, byte zahl)
	{
		if(zahl > -100)
			stb.append(' ');
		if(zahl > -10 && zahl < 100)
			stb.append(' ');
		if(zahl >= 0 && zahl < 10)
			stb.append(' ');
		stb.append(zahl);
	}
}
