package kerberos.packets;


/*
 * Created on Oct 20, 2003
 */

/**
 * 
 * Die Hauptaufgaben des PacketBuilders ist die Serialisierung der Nutzdaten sowie
 * deren Verschlsselung sofern erforderlich (also extern angegeben).
 * 
 * Dieser PaketBuilder erzeugt genau ein Paket. Mit getPaket() lsst es sich auslesen
 * Diese Klasse kennt keine eigenen Schlssel, wei nur, wie man mit einem bergebenen
 * Schlssel verschlsselt.
 * 
 * Der Konstruktor bietet die Mglichkeit, einen Message-Typ zu bergeben.
 * Leerer Konstruktor verwendet automatisch den Typ M_NO_MESSAGE.
 * Die MessageTypen sind in Messages/Message.java definiert.
 * Der MessageTyp ist immer das erste Byte eines Pakets.
 * Dies ist im Kerberos RFC nicht spezifiziert, wird hier aber
 * zur Vereinfachung der Kommunikation verwendet.
 *
 * Die PacketException sollte im Normalfall nie auftreten.
 * Daher wre eine PacketException ein Indiz fr einen
 * Programmier- oder kritischen Systemfehler
 * 
 * @see IPaketReader
 * @see messages.Message
 *
 * @author Christian Morgenstern
 */
public interface IPacketBuilder
{

	/**
	 * Erzeugt einen Nonce-Wert. Dieser wird dann verschlsselt und intern gespeichert
	 * @param schlssel Schlssel im Binrformat 
	 * @return Gibt den erzeugten Noncewert unverschlsselt zurck
	 */
	public long addNonce(byte[] schlssel) throws PacketException;

	/**
	 * Erzeugt einen Nonce-Wert und speichert ihn unverschlsselt
	 * @return erzeugten Nonce-Wert (unverschlsselt)
	 */
	public long addNonceNotCrypt() throws PacketException;

	/**
	 * Fgt einen String hinzu, der nicht verschlsselt wird.
	 * @param string Unverschlsselt hinzuzufgender String 
	 */
	public void addNotCrypt(String string) throws PacketException;
	
	/**
	 * Fgt ein byte-Array hinzu, das nicht verschlsselt wird. 
	 * @param zeichen Unverschlsselt hinzuzufgende Binrdaten 
	 */
	public void addNotCrypt(byte[] zeichen) throws PacketException;

	/**
	 * Fgt einen String hinzu, der mit dem bergebenen Schlssel verschlsselt wird. 
	 * @param string Verschlsselt hinzuzufgender String
	 * @param schlssel Schlssel im Binrformat 
	 */	
	public void add(String string, byte[] schlssel) throws PacketException;

	/**
	 * Fgt ein byte-Array hinzu, das mit dem bergebenen Schlssel verschlsselt wird. 
	 * @param string Verschlsselt hinzuzufgende Binrdaten
	 * @param schlssel Schlssel im Binrformat 
	 */
	public void add(byte[] zeichen, byte[] schlssel) throws PacketException;
	
	/**
	 * Nimmt die Zahl unverschlsselt auf.
	 * @param zahl Unverschlsselt hinzuzufgende Zahl 
	 */
	public void addNotCrypt(long zahl) throws PacketException;

	/**
	 * Verschlsselt die Zahl und nimmt sie in das Paket auf
	 * @param zahl Verschlsselt hinzuzufgende Zahl 
	 * @param schlssel Schlssel im Binrformat 
	 */
	public void add(long zahl, byte[] schlssel) throws PacketException;

	/**
	 * bergibt alle bisher gesammelten Daten. Mehrfachaufruf mglich.
	 * @return Byte-Array mit den gesammelten hinzugefgten Daten. 
	 */
	public byte[] getPaket(); 
}
