/*
 * Created on Oct 20, 2003
 */
package kerberos.packets;

import kerberos.exceptions.PasswortFalschException;

/**
 * Die Nachrichten-Typen finden sich in Messages/Message.java
 * Der Konstruktor bekommt das Paket als Byte-Array bergeben.
 * Die Werte mssen in der gleichen Reihenfolge und Verschlsseltung
 * ausgelesen werden, in der sie auch gespeichert worden sind,
 * Denn es ist keinerlei Typ oder Reihenfolgeninformation im Paket enthalten.
 * Allerdings ist ber den Messagetyp der zugehrige Paketinhalt definiert
 * wodurch dieses in festgelegter Reihenfolge ausgelesen werden kann.
 * 
 * Die PacketException sollte im Normalfall nie auftreten.
 * Daher wre eine PacketException ein Indiz fr einen
 * Programmier- oder kritischen Systemfehler
 * 
 * @see IPaketBuilder
 * @see messages.Message
 * 
 * @author Christian Morgenstern
 */
public interface IPacketReader 
{
	/**
	 * Liest einen verschlsselten Nonce-Wert
	 * @param schlssel Schlssel im Binrformat 
	 * @return Gelesener Nonce-Wert (unverschlsselt)
	 */
	public long readNonce(byte[] schlssel) throws PacketException, PasswortFalschException;

	/**
	 * Liest einen unverschlsselten Nonce-Wert
	 * @return Gelesener Nonce-Wert
	 */
	public long readNonceNonCrypt() throws PacketException;
	
	/**
	 * Liest einen verschlsselten String
	 * @param schlssel Schlssel im Binrformat 
	 * @return Gelesener String (unverschlsselt)
	 */
	public String readString(byte[] schlssel) throws PacketException, PasswortFalschException;

	/**
	 * Liest einen unverschlsselten String
	 * @return Gelesener String
	 */
	public String readStringNonCrypt() throws PacketException;
	
	/**
	 * Liest verschlsselte Binrdaten
	 * @param schlssel Schlssel im Binrformat 
	 * @return Gelesene Binrdaten (unverschlsselt)
	 */
	public byte[] readData(byte[] schlssel) throws PacketException, PasswortFalschException;
	
	/**
	 * Liest unverschlsselte Binrdaten
	 * @return Gelesene Binrdaten
	 */
	public byte[] readDataNonCrypt() throws PacketException;

	/**
	 * Liest verschlsselte Zahl
	 * @param schlssel Schlssel im Binrformat 
	 * @return Gelesene Zahl (unverschlsselt)
	 */
	public long readLong(byte[] schlssel) throws PacketException, PasswortFalschException;

	/**
	 * Liest unverschlsselte Zahl
	 * @return Gelesene Zahl
	 */
	public long readLongNonCrypt() throws PacketException;
	
	/**
	 * Liest den Message-Typ des Paketes.
	 * @return Message-Typ dieses Paketes. Die Typen sind in messages/Message.java spezifiziert
	 */
	public int getMessageType();
	
}
