package kerberos.packets;

import java.io.ByteArrayOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Random;

import kerberos.messages.Message;
import kerberos.net.Passwort;

/**
 * Der PacketBuilder serialisiert die Daten in einem ANS1 hnlichen Format.
 * Daher wird, sofern ein Datentyp eine variable Lnge haben kann,
 * diese vor den jeweiligen Nutzdaten gespeichert
 * 
 * @author Christian Morgenstern
 */
/* (non-Javadoc)
 * @see kerberos.tickets.IPaketBuilder
 */
public class PacketBuilder implements IPacketBuilder
{
	private ByteArrayOutputStream 	outByte;
	private Random 					rnd;

	private static void debugOut( String text )
	{
		PacketReader.debugOutDirekt("PacketBuilder." + text);
	}
		
	/**
     * Default Konstruktor.
     * Erzeugt ein leeres Paket mit dem Typ NO_MESSAGE.
     * Dies ist zum Beispiel sinnvoll fr Aneinanderreihungen von Daten
     * die insgesamt verschlsselt und dann in ein anderes Paket eingebettet werden.
     */
	public PacketBuilder() throws PacketException
	{
		initialize();
		writeByte(Message.M_NO_MESSAGE);
		debugOut("<init> NO_MESSAGE");
	}

	/**
	 * Erzeugung eines Pakets mit festgelegtem Messagetyp.
	 * @param MessageType Nachrichtentyp-Konstante
     * Der Typ wird intern nicht auf Gltigkeit berprft oder verwendet.
	 */
	public PacketBuilder( int MessageType ) throws PacketException
	{	
		// assertions knnen aufgrund der PDA Kompatibilitt nicht verwendet werden
		//assert( MessageType >= Message.M_FIRST && MessageType <= Message.M_LAST );
		
		initialize();
		writeByte(MessageType);
		debugOut("<init> " + Message.getString(MessageType ));
	}
		
	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#add(long, byte[])
	 */
	public void add(long zahl, byte[] schlssel) throws PacketException
	{
		debugOut("add( " + zahl + ", " + Passwort.toZahl(schlssel ));
		addNotCrypt( encrypt( longToBytes( zahl ), schlssel ) );	
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#add(java.lang.String, byte[])
	 */
	public void add(String string, byte[] schlssel) throws PacketException
	{
		debugOut("add( " + string + ", " + Passwort.toZahl(schlssel ));
		addNotCrypt( encrypt( string.getBytes(), schlssel ) );	
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#addNonce(byte[])
	 */
	public long addNonce(byte[] schlssel) throws PacketException
	{
		long nonce = generateNonce();
		add( nonce, schlssel );
		debugOut("addNonce( " + Passwort.toZahl(schlssel) + " ret: " + nonce );

		return nonce;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#add(byte[], byte[])
	 */
	public void add(byte[] zeichen, byte[] schlssel) throws PacketException
	{
		debugOut("add( " + Passwort.toZahl(zeichen) + ", " + Passwort.toZahl(schlssel ));
		addNotCrypt( encrypt( zeichen, schlssel ) );	
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#addNonceNotCrypt()
	 */
	public long addNonceNotCrypt() throws PacketException
	{
		long nonce = generateNonce();
		addNotCrypt( nonce );
		debugOut("addNonceNotCrypt() ret: " + nonce );
		return nonce;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#addNotCrypt(byte[])
	 */
	public void addNotCrypt(byte[] zeichen) throws PacketException
	{
		debugOut("addNotCrypt( " + Passwort.toZahl(zeichen) + ")" );
		addNotCrypt( zeichen.length );
		writeBytes( zeichen );
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#addNotCrypt(long)
	 */
	public void addNotCrypt(long zahl) throws PacketException
	{
		debugOut("addNotCrypt( " + zahl + ")" );
		writeBytes( longToBytes( zahl ) );
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#addNotCrypt(java.lang.String)
	 */
	public void addNotCrypt(String string) throws PacketException
	{
		debugOut("addNotCrypt( " + string + ")" );
		addNotCrypt( string.getBytes() );
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketBuilder#getPaket()
	 */
	public byte[] getPaket()
	{
		debugOut("getPaket()" );
		return outByte.toByteArray();
	}
	
	private void initialize()
	{
		outByte = new ByteArrayOutputStream();
		rnd = new Random();
	}
	
	private void writeByte(int data) throws PacketException
	{
		if ( data > 255 )
			throw new PacketException(new Exception("Invalid datasize"));
		
		byte bdata[] = new byte[1];
		bdata[0] = new Integer(data).byteValue();
		try{ outByte.write( bdata ); }
		catch( Exception e ){ throw new PacketException( e ); };
	}
	
	private void writeBytes(byte[] data) throws PacketException
	{
		try{ outByte.write( data ); }
		catch( Exception e ){ throw new PacketException( e ); };
	}
	
	private long generateNonce()
	{
		return rnd.nextLong();
	}

	private byte[] encrypt(byte[] data, byte[] key) throws PacketException
	{
		// Checksum merken
		byte[] checksum = getChecksum( data );
		//assert( checksum.length == 16 );

		// Verschlsseln
		DES desCipher = new DES( key );

		// Richtige Blocksize
		int blockPadding = 8 - ( data.length % desCipher.cipherBlockSize() );
		if( blockPadding == 8 )
			blockPadding = 0;
			
		byte[] toCryptData = new byte[ data.length + blockPadding ];
		byte[] crypted = new byte[ toCryptData.length ]; 
		for( int i = 0; i < data.length; i++ )
			toCryptData[i] = data[i];

		// Nun pro Block verschlsseln
		for( int i = 0; i < toCryptData.length; i = i + desCipher.cipherBlockSize() )
			desCipher.encrypt( toCryptData, i, crypted, i );

		// Zusammenfgen
		byte[] result = new byte[checksum.length + crypted.length + 1];
		for( int i = 0 ; i < checksum.length ; i++ )
			result[i] = checksum[i];
			
		result[ checksum.length ] = (byte)blockPadding;
		for( int i = 0 ; i < crypted.length ; i++ )
			result[checksum.length + 1 + i] = crypted[i];

		debugOut("encrypted " + result.length + " : " + Passwort.toZahl(result) );

		return result;
	}
	
	private byte[] getChecksum(byte data[]) throws PacketException
	{
		try
		{
			MessageDigest md = MessageDigest.getInstance("MD5");
			return md.digest(data);
		}
		catch (NoSuchAlgorithmException e)
		{
			throw new PacketException(e);
		}

	}
	
	private static byte[] longToBytes(long l)
	{
		byte byte8[] = new byte[8];
		for( int i = 7 ; i >= 0 ; i-- ) 
			byte8[i] = (byte)(int)(l >> ( 56 - i * 8 ) & 255L);
			
		return byte8;
	}
	

}