package kerberos.packets;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import util.logger.Logger;
import util.logger.NullLogger;
import util.logger.SystemOutLogger;

import kerberos.exceptions.PasswortFalschException;
import kerberos.net.Passwort;

/**
 * Der PacketBuilder deserialisiert die Daten in einem ANS1 hnlichen Format.
 * Daher wird, sofern ein Datentyp eine variable Lnge haben kann,
 * diese vor den jeweiligen Nutzdaten ausgelesen um die jeweiligen Nutzdaten
 * komplett zu lesen.
 * 
 * @author Christian Morgenstern
 */
/* (non-Javadoc)
 * @see kerberos.tickets.IPaketReader
 */
public class PacketReader implements IPacketReader
{	
	private ByteArrayInputStream 	inByte;
	private int 					messageType;
	
	/**
	 * Dieser Stream wird fr debug-Ausgaben genutzt. Er kann durch einen anderen ersetzt werden, auch durch null.
	 */
	private static Logger logger = null;//System.out;

	/**
	 * PackageBuilder und PackageReader geben keine debug-Ausgaben mehr aus. 
	 */
	public static void debugOff()
	{
		logger = NullLogger.get();
	}

	public static void debugOn()
	{
		logger = new SystemOutLogger();
	}

	private static void debugOut( String text )
	{
		debugOutDirekt("PacketReader." + text);		
	}

	/**
	 * Gibt diesen String direkt auf dem aktuellen systemOut aus (oder eben gar nicht)
	 * @param text
	 */
	protected static void debugOutDirekt( String text )
	{
		if(logger != null)
			logger.log(text, 8);		
	}

	/**
	 * Erzeugung eines Readers auf ein Paket in Binrform
	 * @param data Paket in Binrform wie es von PacketBuilder->getPacket() erzeugt wurde.
	 */
	public PacketReader( byte[] data ) throws PacketException
	{
		inByte = new ByteArrayInputStream( data );
		messageType = new Byte(readBytes(1)[0]).intValue(); 

		debugOut("<init> " + messageType + " - "+ Passwort.toZahl(data) );
		
		//assert( messageType >= Message.M_FIRST && messageType <= Message.M_LAST );
	}
	
	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#getMessageType()
	 */
	public int getMessageType()
	{
		return messageType;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readData(byte[])
	 */
	public byte[] readData(byte[] schlssel) throws PacketException, PasswortFalschException
	{
		//debugOut("readData( " + Passwort.toZahl(schlssel) + ")" );
		byte result[] = decrypt( readDataNonCrypt(), schlssel );
		//debugOut("readData result" + Passwort.toZahl(result) );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readLong(byte[])
	 */
	public long readLong(byte[] schlssel) throws PacketException, PasswortFalschException
	{
		debugOut("readLong( " + Passwort.toZahl(schlssel) + ")" );
		long result = bytesToLong( decrypt( readDataNonCrypt(), schlssel ) );
		debugOut("readLong result " + result );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readNonce(byte[])
	 */
	public long readNonce(byte[] schlssel) throws PacketException, PasswortFalschException
	{
		debugOut("readNonce( " + Passwort.toZahl(schlssel) + ")" );
		long result = readLong( schlssel );
		debugOut("readNonce result " + result );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readString(byte[])
	 */
	public String readString(byte[] schlssel) throws PacketException, PasswortFalschException
	{
		debugOut("readString( " + Passwort.toZahl(schlssel) + ")" );
		String result = new String( decrypt( readDataNonCrypt(), schlssel ) );
		debugOut("readString result " + result );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readDataNonCrypt()
	 */
	public byte[] readDataNonCrypt() throws PacketException
	{
		debugOut("readDataNonCrypt()" );
		byte result[] = readBytes( (int)readLongNonCrypt() );
		debugOut("readDataNonCrypt result" + Passwort.toZahl(result) );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readLongNonCrypt()
	 */
	public long readLongNonCrypt() throws PacketException
	{
		debugOut("readLongNonCrypt()" );
		long result = bytesToLong( readBytes(8) );	// long ist 8 byte
		debugOut("readLongNonCrypt result " + result );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readNonceNonCrypt()
	 */
	public long readNonceNonCrypt() throws PacketException
	{
		debugOut("readNonceNonCrypt()" );
		long result = readLongNonCrypt();
		debugOut("readNonceNonCrypt result " + result );
		return result;
	}

	/* (non-Javadoc)
	 * @see kerberos.tickets.IPaketReader#readStringNonCrypt()
	 */
	public String readStringNonCrypt() throws PacketException
	{
		debugOut("readStringNonCrypt()" );
		String result = new String( readDataNonCrypt() );
		debugOut("readStringNonCrypt result " + result );
		return result;
	}
	
	private byte[] readBytes( int count ) throws PacketException
	{
		byte[] read = new byte[count];
		if( inByte.read( read, 0, count) != count )
			throw new PacketException( new IOException("Keine Daten mehr zum lesen") );
		
		return read;
	}
	
	private byte[] decrypt(byte[] data, byte[] key) throws PacketException, PasswortFalschException
	{
		debugOut("decrypt( " + Passwort.toZahl(data) + ", " + Passwort.toZahl(key) + ")" );
		//assert( data.length >= 16 );
		
		byte[] checksumR = new byte[16]; 
		byte[] decdata = new byte[data.length - 17];
		
		// Checksum abtrennen
		for( int i = 0 ; i < 16 ; i++ )
			checksumR[i] = data[i];
		
		int blockPadding = data[16];
		for( int i = 0 ; i < data.length - 17 ; i++ )
			decdata[i] = data[i+17];

		byte[] decrypted = new byte[ decdata.length ];
		
		// Lnge prfen				
		DES desCipher = new DES( key );			
		if( decdata.length % desCipher.cipherBlockSize() != 0 )
			throw new PacketException( new Exception("Datenblock hat falsche Lnge") );
			 
		// Pro Block entschlsseln
		for( int i = 0; i < decdata.length; i = i + desCipher.cipherBlockSize() )
			desCipher.decrypt( decdata, i, decrypted, i );
		
		byte[] result = new byte[ decrypted.length - blockPadding ];
		for( int i = 0; i < result.length; i++ )
			result[i] = decrypted[i];
		
		// Checksum prfen	
		byte[] checksumD = getChecksum( result );
		//assert( checksumD.length == 16 );
		
		for( int i = 0 ; i < 16 ; i++ )
			if( checksumR[i] != checksumD[i] )
				throw new PasswortFalschException();

		debugOut("decryptResult - " + Passwort.toZahl(result) + ")" );
		
		return result;
	}

	private byte[] getChecksum(byte data[]) throws PacketException
	{
		try
		{
			MessageDigest md = MessageDigest.getInstance("MD5");
			return md.digest(data);
		}
		catch (NoSuchAlgorithmException e)
		{
			throw new PacketException(e);
		}

	}

	private static long bytesToLong(byte byte8[])
	{
		long result = 0;
		for( int i = 7 ; i >= 0 ; i-- ) 
			result += ((long)(byte8[i] >= 0 ? byte8[i] : 256 + byte8[i])) << ( 56 - i * 8 );

		return result;
	}
}
