/*
 * Created on 01.11.2003
 *
 */
package kerberos.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import kerberos.exceptions.PasswortDestroyedException;
import kerberos.net.Passwort;
import kerberos.server.keyaccess.AbstractKeyAccessor;
import kerberos.server.keyaccess.IKeyAccessor;


/**
 * KDC ist die Hauptklasse die den KDC startet und inizialisiert.<br>
 * Ausserdem kmmert sie sich danach um die Konsoleneingaben.
 * @author Sven Elvers
 *
 */
public class KDC {
	
	private KDCServer serverThread;
	private IKeyAccessor userBank;
	
	private static final int CMD_INVALID = 0;
	private static final int CMD_EXIT = 1;
	private static final int CMD_INFO = 2;
	private static final int CMD_ADDUSER = 3;
	private static final int CMD_HELP = 4;
	private static final int CMD_DELUSER = 5;
	private Map commands;
	
	/**
	 * Initialisiert eine Map mit allen Kommandos
	 */
	public KDC()
	{
		commands = new HashMap();
		commands.put( "help", new Integer( CMD_HELP ) );
		commands.put( "exit", new Integer( CMD_EXIT ) );
		commands.put( "info", new Integer( CMD_INFO ) );
		commands.put( "adduser", new Integer( CMD_ADDUSER ) );
		commands.put( "deluser", new Integer( CMD_DELUSER ) );
	}
	
	/**
	 * startet den KDC und berprft vorher, ob die bentigten
	 * Properties und ob eine Schlsselbank vorhanden sind.
	 */
	public void start()
	{ 
		if (KDCProperties.getInstace() == null) return;
		
		userBank = AbstractKeyAccessor.getKeyAccessor();
		if (userBank == null) return;
		if (userBank.getKey("TGS") == null)
			if(!createPrinzipalTGS()) {
				System.out.println("Couldn't creat Prinzipal TGS!");
				System.out.println("See log.");
				return;
			}
		
		serverThread = new KDCServer();
		serverThread.start();
		
		try {
			handleRequests();
		} catch (IOException e) {
			KDCLogger.log("Inputstream is closed");
			KDCLogger.log(e);
		}
		
		if(!serverThread.isInterrupted() && serverThread.isAlive())  serverThread.interrupt();

		System.out.println("KDC is down");
	}

	/**
	 * fragt nach einen Passwort fr den Prinzipal TGS
	 * @return true, wenn der user TGS existiert
	 */
	private boolean createPrinzipalTGS() {
		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
		try {
			System.out.print("Insert a password for the TGS: ");
			String line = input.readLine().trim();
			Passwort pass = new Passwort(line);
			return AbstractKeyAccessor.getKeyAccessor().addToken("TGS",pass.getSchlssel());
		} catch (PasswortDestroyedException e) {
			KDCLogger.log(e);
		} catch (IOException e) {
			KDCLogger.log(e);
		} catch (Exception e) {
			KDCLogger.log(e);
		}
		
		return false;
	}

	/**
	 * liest die Konsolen-Eingaben aus gibt sie an handleRequest()
	 * weiter bis der Befehl zum Beenden des KDC eingegeben wurde 
	 * @throws IOException
	 * wenn beim Arbeiten mit den Streams ein Fehler auftritt
	 */
	private void handleRequests() throws IOException {
		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
		String token;
		boolean shutdown = false;

		while (!shutdown) {
			System.out.println();
			System.out.print("KDC-Console: ");
			
			token = input.readLine().trim();
			if (token.length() > 0){
				shutdown = handleRequest(token);
			}
		}
		System.out.println("KDC is shutting down...");
	}

	/**
	 * Arbeitet einen Befehl ab, der in die Konsole eingegeben wurde
	 * @param token
	 * ein Befehl sammt Parameter
	 * @return
	 * true, wenn der KDC beendet werden soll
	 */
	private boolean handleRequest(String token) {
		StringTokenizer tok = new StringTokenizer(token);
		String cmdstring = tok.nextToken();
		int cmdnumber = CMD_INVALID;
		for (Iterator iter = commands.entrySet().iterator(); iter.hasNext();)
		{
			Map.Entry element = (Map.Entry)iter.next();
			if( ((String)element.getKey()).compareTo( cmdstring ) == 0 )
			{
				cmdnumber = ((Integer)element.getValue()).intValue(); 
				break;	
			}
		}
			
		switch( cmdnumber )
		{
			case CMD_INVALID:
				System.out.println("Unknown command");
				break;

			case CMD_EXIT:
				return true;

			case CMD_INFO:
				if (serverThread.isAlive()) {
					serverThread.printInfoOn(System.out);
					} else System.out.println("Kerberos-Server is not running");
				break;

			case CMD_HELP:
				StringBuffer helpbuf = new StringBuffer();
				for (Iterator iter = commands.entrySet().iterator(); iter.hasNext();)
				{
					helpbuf.append(	(String)((Map.Entry)iter.next()).getKey() + " " );
				}
				System.out.println( helpbuf );
				break;

			case CMD_ADDUSER:
			{
				if( tok.countTokens() != 2 )
				{
					System.out.println("Usage: " + cmdstring + " <username> <password>");
					break;						
				} 
				
				String username = tok.nextToken();
				Passwort pw = new Passwort( tok.nextToken().toCharArray() );
				boolean result = false;
				try{
					result = userBank.addToken( username, pw.getSchlssel() ); 
				}
				catch( Exception e ){
					KDCLogger.log(e);
				}
				if( result )
					System.out.println("User added");
				else
					System.out.println("Unable to add user");
				break;
			}

			case CMD_DELUSER:
			{
				if( tok.countTokens() != 1 )
				{
					System.out.println("Usage: " + cmdstring + " <username>");
					break;						
				} 
				
				String username = tok.nextToken();
				boolean result = false;
				try{
					result = userBank.removeToken( username ); 
				}
				catch( Exception e ){
					KDCLogger.log(e);
				}
				if( result )
					System.out.println("User removed");
				else
					System.out.println("Unable to remove user");
				
				break;
			}

			default:
				System.out.println("Invalid command, try \"help\"");
				break;
			
		}
			
		return false;	
	}

	/**
	 * main-Methode um den KDC zu starten
	 * @param args
	 * werden ignoriert, da keine bentigt werden
	 */
	public static void main(String[] args) {
		KDC kdc = new KDC();
		kdc.start();
	}
}
