/*
 * Created on 03.11.2003
 *
 */
package kerberos.server;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.KeyGenerator;

import kerberos.messages.M_ERROR;
import kerberos.messages.M_KDC_AS_REQ;
import kerberos.messages.M_KDC_REQ;
import kerberos.messages.M_KDC_TGS_REQ;
import kerberos.messages.Message;
import kerberos.packets.PacketException;
import kerberos.server.keyaccess.AbstractKeyAccessor;
import kerberos.server.keyaccess.IKeyAccessor;

/**
 * Diese Klasse Verwaltet ein Client-KDC Verbindung.
 * @author Sven Elvers
 */
public class KDCConnection extends Thread {

	//Schlssel fr die KDC.properties
	public static final String TICKET_DURABILITY_HH_KEY = "<TicketDurability-hours>";
	public static final String TICKET_DURABILITY_MM_KEY = "<TicketDurability-minutes>";
	public static final String TICKET_DURABILITY_SS_KEY = "<TicketDurability-seconds>";

	//Klassenkonstanten
	public static final int TICKET_DURABILITY_DEFAULT = (4 * 3600000); //4 hours

	//Instancevariablen
	protected KDCSocket socket;
	protected IConnectionListener eventlistener;

	/**
	 * @param socket
	 * @param userBank
	 */
	public KDCConnection(KDCSocket socket) {
		super();
		this.socket = socket;
	}

	/**
	 * Wird durch die Methode <code>start()</code> aufgerufen.<br>
	 * Es wird eine Anfrage des Clients bearbeitet und danach 
	 * wird die Verbindung wieder geschlossen.
	 * @see java.lang.Thread#run()
	 */
	public void run() {
		super.run();

		KDCLogger.log("Bearbeite Verbindung in KDCConnection");

		try {
			Message m = socket.getMessage();
			KDCLogger.log("KDCConnection: Message gelesen, wird gleich bearbeitet");
			handleMessage(m);
		} catch (Exception e) {
			if (!this.isInterrupted()) {
				KDCLogger.log(e);
			}
		}
		
		KDCLogger.log("KDCConnection: Message behandelt, gleich Socket schlieen");
		try {
			if (socket != null)
				socket.close();
		} catch (IOException e) {
			KDCLogger.log(e);
		}
		socket = null;
		eventlistener.lostConnection(this);

		KDCLogger.log("Verbindung in KDCConnection ist geschlossen");

	}

	/**
	 * Schaut nach welche Message eingegangen ist und bearbeitet diese dann.<br>
	 * Falls er mit der Message nichts anfangen kann, wird eine ErrorMessage an<br>
	 * den Client zurckgeschickt.
	 * @param message
	 * eine KDC-Anfrage
	 */
	private void handleMessage(Message message) {
		if (message instanceof M_KDC_REQ) {
			KDCLogger.log("KDCConnection.handleMessage(): Username=" + ((M_KDC_REQ) message).getUsername());
		} else {
			KDCLogger.log("KDCConnection.handleMessage(): keine Anfrage an KDC, sondern ein " + message.getClass());
		}
		try {
			if (message instanceof M_KDC_AS_REQ)
				handleLoginRequest((M_KDC_AS_REQ) message);
				
			else if (message instanceof M_KDC_TGS_REQ)
				handleTicketRequest((M_KDC_TGS_REQ) message);
			
			else socket.sendError(message.getType(), M_ERROR.ERR_MSG_TYPE);
				 
		} catch (Exception e) {
			try {
				socket.sendError(message.getType(), M_ERROR.ERR_MODIFIED);
			} catch (Exception e1) {}
			KDCLogger.log(e);
		}
	}

	/**
	 * Anhand der Anfrage wird der Schlssel fr den Zielserver herausgesucht sowie<br>
	 * ein Sitzungsschlssel erstellt. Mit diesen Werten und der Lebensdauer eines Tickets<br>
	 * wird dann eine Antwort-Nachricht an den Client geschickt.
	 * @param message
	 * eine Ticket-Anfrage
	 * @throws IOException
	 * wenn die Nachricht nicht mehr verschickt werden kann.
	 */
	private void handleTicketRequest(M_KDC_TGS_REQ message)
		throws IOException, PacketException, Exception {
			
		try {
			IKeyAccessor keys = AbstractKeyAccessor.getKeyAccessor();

			byte[] sitzungsSchlssel =
				KeyGenerator.getInstance("DES").generateKey().getEncoded();
			byte[] serverSchlssel = keys.getKey(message.getZieldienstname());

			if(sitzungsSchlssel == null) 
				throw new Exception("Sitzungsschlssel konnte nicht generiert werden");

			if (serverSchlssel != null) {
//				KDCLogger.log("KDCConnection.handleTicketRequest: ServerSchlssel ist != null");
				socket.sendServerTicket(
					message,
					ticketDurability(),
					serverSchlssel,
					sitzungsSchlssel);
			} else {
				KDCLogger.log("KDCConnection.handleTicketRequest: kein Zieldienst mit dem Namen " + message.getZieldienstname());
				socket.sendError(
					Message.M_KDC_TGS_REQ,
					M_ERROR.ERR_S_PRINCIPAL_UNKNOWN);
			}
		} catch (NoSuchAlgorithmException e) {
			KDCLogger.log(e);
		}
	}

	/**
	 * Anhand der Anfrage wird der Schlssel fr den User und dem TGS herausgesucht sowie<br>
	 * ein Sitzungsschlssel erstellt. Mit diesen Werten und der Lebensdauer eines Tickets<br>
	 * wird dann eine Antwort-Nachricht an den Client geschickt. 
	 * @param message
	 * eine Login-Anfrage
	 * @throws IOException
	 * wenn die Nachricht nicht mehr verschickt werden kann.
	 */
	private void handleLoginRequest(M_KDC_AS_REQ message)
		throws IOException, PacketException, Exception {
		try {
			IKeyAccessor keys = AbstractKeyAccessor.getKeyAccessor();

			byte[] tgsSchlssel = keys.getKey("TGS");
			byte[] sitzungsSchlssel =
				KeyGenerator.getInstance("DES").generateKey().getEncoded();
			// TODO Hinweise zu DES
			byte[] userSchlssel = keys.getKey(message.getUsername());

			if(sitzungsSchlssel == null) throw new Exception("Sitzungsschlssel konnte nicht generiert werden");
			if(tgsSchlssel == null) throw new Exception("Es gibt keinen Schlssel fr den TGS");

			if (userSchlssel != null)
				socket.sendTgsTicket(
					message,
					ticketDurability(),
					tgsSchlssel,
					sitzungsSchlssel,
					userSchlssel);
			else
				socket.sendError(
					Message.M_KDC_AS_REQ,
					M_ERROR.ERR_C_PRINCIPAL_UNKNOWN);
		} catch (NoSuchAlgorithmException e) {
			KDCLogger.log(e);
		}
	}

	/**
	 * @return
	 * den Wert, der in der KDC.properties steht. Wenn dieser 0 ist oder
	 * nicht angegeben, wird der Wert der Konstante <code>TICKET_DURABILITY_DEFAULT</code>
	 * zurckgegeben.
	 */
	private long ticketDurability() {
		KDCProperties p = KDCProperties.getInstace();
		int time = 0;
		try {
			int hh = p.getInt(TICKET_DURABILITY_HH_KEY);
			int mm = p.getInt(TICKET_DURABILITY_MM_KEY);
			int ss = p.getInt(TICKET_DURABILITY_SS_KEY);

			time = ((((hh * 60) + mm) * 60 + ss) * 1000);
		} catch (NumberFormatException e) {
			KDCLogger.log(e);
		}

		if (time > 0)
			return time;
		return TICKET_DURABILITY_DEFAULT;
	}

	/**
	 * Hier kann ein connectionListener bergeben werden, dem beim Beenden des Threads,<br>
	 * ein LostConnection-Event bergeben wird.
	 * @param listener
	 * der ConnectionListener
	 */
	public void addEventListener(IConnectionListener listener) {
		this.eventlistener = listener;
	}

	/**
	 * Setzt eine interrupted Flag fr den Thread, dass dieser sich beenden soll.<br>
	 * Zustzlich beendet er die Verbindung zum Client und lscht die Referenz<br>
	 * auf die Datenbank mit den Schlsseln.
	 * @see java.lang.Thread#interrupt()
	 */
	public void interrupt() {
		super.interrupt();

		try {
			if (socket != null)
				socket.close();
		} catch (IOException e) {}
	}

}
