/*
 * Created on 10.01.2004
 */
package kerberos.server;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Properties;

/**
 * Singelton-Pattern<br>
 * Eine Instance dieser Klasse wird ber die Methode <code>getInstance()</code> geholt
 * und mit den Gettern knnen dann Parameter ausgelesen werden.<br>
 * Wenn eine neue Instance erstellt wird, holt diese sich die Informationen <br>
 * aus der KDC.properties Datei die im selben Verzeichnis wie das kerberos Package liegt.
 * Die KDC.properties wird nach allen Schlsseln, die bentigt werden, untersucht und kontrolliert.
 * Wenn die ein Fehler auftritt, wird eine Fehlermeldung ausgegeben und null zurckgegeben.
 * @author Sven Elvers
 */
public class KDCProperties {

	//Klassenkonstanten
	public static final String DEFAULT_FILEPATH = "../../KDC.properties";

	//Klassenvariablen
	private static KDCProperties Instance = null;

	//Instancevariablen
	private Properties prop = null;

	/**
	 * ist wegen Singeltonpatern als private deklariert
	 * @throws Exception
	 * wenn nicht alle bentigten Informationen da sind
	 * oder wenn die Datei nicht gefunden wurde
	 */
	private KDCProperties() throws Exception {
		Properties p = new Properties();
		URL url = this.getClass().getResource(DEFAULT_FILEPATH);
		if(url == null)
			throw new Exception("Die Datei KDC.properties fehlt");
		
		InputStream stream = url.openStream();
		load(p,stream);
		stream.close();
		
		String[] s = {""};
		if (!valid(p,s))
			throw new Exception("Die Datei KDC.properties ist Fehlerhaft:\n" + s[0]);
			
		prop = p;
	}

	/**
	 * Schlssel sind am Anfang einer Zeile stehende Strings,
	 * die mit &lt; anfangen und mit &gt; enden.<br>
	 * Werte sind Strings die hinter Schlsseln in der gleichen Zeile stehen.<br>
	 * Zeilen, die mit # beginnen sind Kommentare
	 * @param p Properties-Objekt, das gefllt werden soll
	 * @param stream InputStream mit den Properties
	 */
	private void load(Properties p, InputStream stream) throws IOException {
		BufferedReader in = new BufferedReader(new InputStreamReader(stream));
		while (in.ready()){
			String line = in.readLine().trim();
			if (line.length() > 0 && !line.startsWith("#")){
				int start = line.indexOf('<');
				int end = line.indexOf('>') + 1;
				if (start >= 0 && end > 1 && end > start){
					String key = line.substring(start,end);
					String value = line.substring(end,line.length()).trim();
					p.put(key,value);
				}
			}
		}
	}

	/**
	 * gibt null zurck, wenn kein Objekt existiert und auch nicht erstellt werden kann
	 * @return Singelton der Klasse
	 */
	public static KDCProperties getInstace(){
		if (Instance == null) {
			try {
				Instance = new KDCProperties();
			} catch (Exception e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}
		
		return Instance; 
	}
	
	/**
	 * schaut nach ob alle bentigten Schlssel da sind und 
	 * ob der jeweils angegebene Parameter korrekt ist.
	 * @param p
	 * Ein Properties-Objekt das berprft werden soll.
	 * @param s
	 * ein StringArray der Lnge 1.
	 * Dieser wird mit Fehlerhinweisen in dieser Methode gefllt.
	 * @return
	 * <li> true, wenn alles i.O. ist </li>
	 * <li> false, wenn Daten fehlen oder nicht korrekt sind </li> 
	 */
	private static boolean valid(Properties p, String[] s) {
		String str = "";
		String keyAccessorParam = "";
		
		//<KeyAccessor-KeyDatabase>
		if (p.containsKey("<KeyAccessor-KeyDatabase>")){
			String[] params = {"File"};
			keyAccessorParam = p.getProperty("<KeyAccessor-KeyDatabase>");
			int index = -1;
			
			for (int i = 0; i < params.length; i++)
				if (params[i].equalsIgnoreCase(keyAccessorParam)) index = i;
			
			if (index < 0)
				str += "'" + keyAccessorParam + "' is unknown for '<KeyAccessor-KeyDatabase>'! \n" +
					"\tOptions: File\n";
					
		} else str += "Key '<KeyAccessor-KeyDatabase>' is missing in KDC.properties!\n";
		
		//<KeyFileAccessor-File>
		if (keyAccessorParam.equalsIgnoreCase("File")){
			if (p.containsKey("<KeyFileAccessor-File>")){
				String param = p.getProperty("<KeyFileAccessor-File>");
				File file = new File(param);
				if (!file.exists()) 
					str += "'<KeyFileAccessor-File>' File:'" + param + "' does not exist!\n";
			}
			else  str += "Key '<KeyFileAccessor-File>' is missing in KDC.properties!\n";
		}
		
		s[0] = str;
		if(s[0].length() > 0)
			return false;
		return true;
	}

	/**
	 * holt einen Parameter zu einem Schlssel
	 * und gibt ihn als String aus
	 * @param key
	 * @return String
	 */
	public String getString(String key) {
		return prop.getProperty(key);
	}

	/**
	 * holt einen Parameter zu einem Schlssel
	 * und gibt ihn als int aus
	 * @param key
	 * @return int
	 * @throws NumberFormatException, wenn aus dem Wert kein int generiert werden kann.
	 */
	public int getInt(String key) throws NumberFormatException {
		return Integer.parseInt(prop.getProperty(key));
	}

}
