/*
 * Created on Oct 29, 2003
 */
package kerberos.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

import kerberos.exceptions.ArrayZuLangException;
import kerberos.exceptions.ClientnameFalschException;
import kerberos.exceptions.PasswortFalschException;
import kerberos.exceptions.UngltigerMessageTypException;
import kerberos.messages.M_KDC_AS_REQ;
import kerberos.messages.M_KDC_TGS_REQ;
import kerberos.messages.Message;
import kerberos.messages.MessageBuilder;
import kerberos.packets.PacketException;
import sun.net.ConnectionResetException;
import util.NetUtil;

/**
 * @author Stephan Koops
 */
public class KDCSocket
{
	/** Speichert die maximale Lnge der empfangenen Messages */
	public static final int MAX_MESSAGE_LENGTH = 256*256;
	
	protected Socket socket;
	protected OutputStream outputStream;
	protected InputStream inputStream;
	
	/**
	 * Dieser Schlssel ist der Sitzungsschlssel zwischen Client und KDC
	 */
	protected byte[] sitzungsschlssel;
	
	protected byte[] serverSocketSchlssel = null;
	
	/**
	 * Dieser Konstruktor wird vom KDCServerSocket.accept() aufgerufen
	 * @param socket
	 * @see KDCServerSocket#accept()
	 */
	void _KDCSocket(Socket socket, byte[] sitzungsschlssel) throws IOException 
	{
		this.socket = socket;
		this.outputStream = socket.getOutputStream();
		this.inputStream = socket.getInputStream();
		this.sitzungsschlssel = sitzungsschlssel;
	}
	
	KDCSocket(Socket socket, byte[] serverSocketSchlssel)
	{
		this.socket = socket;
		this.serverSocketSchlssel = serverSocketSchlssel;
	}
	
	protected void sendPaket(byte[] paket) throws IOException
	{
		NetUtil.writeArray(this.socket, paket);
	}

	/**
	 * Sendet ein TGS-Ticket zurck
	 * @param reqest
	 * @param tgsSchlssel
	 * @param gltigkeitsdauer in Millisekunden; Das Ticket gilt ab sofort.
	 * @param userschlssel
	 */
	public void sendTgsTicket(M_KDC_AS_REQ request, long gltigkeitsdauer, byte[] tgsSchlssel, byte[] sitzungsschlssel, byte[] userschlssel) throws IOException, PacketException
	{
		byte[] reply = MessageBuilder.currentBuilder().buildKdcAuthLoginOK(
					request.getNonce(),
					request.getUsername(),
					tgsSchlssel,
					"KDC",
					System.currentTimeMillis(),
					System.currentTimeMillis() + gltigkeitsdauer,
					userschlssel,
					sitzungsschlssel);
		this.sendPaket(reply);
	}

	/**
	 * 
	 * @param request
	 * @param gltigkeitsdauer
	 * @param tgsSchlssel
	 * @param sitzungsschlssel_C_S zwischen Client und Ziel-Server
	 */
	public void sendServerTicket(M_KDC_TGS_REQ request, long gltigkeitsdauer, byte[] serverschlssel, byte[] sitzungsschlssel_C_S) throws IOException, PacketException
	{
		String user = new String(request.getUsername());
		String server = request.getServerName();
			
		byte[] reply = MessageBuilder.currentBuilder().buildTGSLoginOK(
					sitzungsschlssel_C_S,
					request.getNonce(),
					user,
					server,
					System.currentTimeMillis(),
					System.currentTimeMillis() + gltigkeitsdauer,
					serverschlssel,
					this.sitzungsschlssel);
		this.sendPaket(reply);
	}

	/**
	 * Sendet eine Fehlermeldung an den Absender.
	 * @see MessageBuilder#buildError(int,int)
	 * @throws IOException Wenn es ein Netzwerkproblem gab.
	 */
	public void sendError(int messageType, int errorcode) throws IOException, PacketException
	{
		NetUtil.sendError(this.socket, messageType, errorcode);
	}
	
	/**
	 * Liest die nchste Message aus. Wenn die bertragenen Message zu lang war, dann wird
	 * eine Fehlermeldung zurckgeschickt und diese Methode gar nicht verlassen.
	 * @return die nchste erfolgreich gelesene Nachricht
	 * @throws PasswortFalschException
	 * @throws ClientnameFalschException
	 * @throws PacketException
	 * @throws UngltigerMessageTypException
	 * @throws ConnectionResetException
	 * @throws IOException
	 */
	public Message getMessage() throws PasswortFalschException, ClientnameFalschException, PacketException, UngltigerMessageTypException, ConnectionResetException, ArrayZuLangException, IOException
	{
		String methodenname = "KDCSocket.getMessage()";
		if(this.sitzungsschlssel == null)
		{
			return this.schlsselLesen();
		}
		System.out.print(methodenname+": ");
		byte[] daten;
		for(;;)
		{
			daten = NetUtil.readByteArray(socket, MAX_MESSAGE_LENGTH);
			return MessageBuilder.currentBuilder().readMessage(daten, sitzungsschlssel);
		}
	}
	
	/**
	 * @return Gibt die gelesene Message zurck 
	 */
	private Message schlsselLesen() throws PasswortFalschException, ClientnameFalschException, UngltigerMessageTypException, PacketException
	{
		Message ersteNachricht;
		String methodenname = "KDCSocket.readSchluessel()";
		boolean debug = false;
		try {
					
			byte[] daten;
			daten = NetUtil.readByteArray(socket, KDCSocket.MAX_MESSAGE_LENGTH);
			if(debug)
				KDCLogger.log(methodenname+": Anfrage des Clients gelesen");
			if(debug)
				KDCLogger.log(methodenname+": Antwort entschlsseln");
			ersteNachricht = MessageBuilder.currentBuilder().readMessage(daten, this.serverSocketSchlssel);
			if(debug)
				KDCLogger.log(methodenname+": Paket entschlsselt");
			if (ersteNachricht instanceof M_KDC_AS_REQ)
			{
				if(debug)KDCLogger.log(methodenname+": Verbindung angenommen !");
				return ersteNachricht;
			}
			if (ersteNachricht instanceof M_KDC_TGS_REQ)
			{
				M_KDC_TGS_REQ tgsReq = (M_KDC_TGS_REQ)ersteNachricht;
				this.sitzungsschlssel = tgsReq.getSchluessel_C_TGS();
				return ersteNachricht;
			}
			if(ersteNachricht == null)
				KDCLogger.log(methodenname+": Nachricht ist null");
			else
				KDCLogger.log(methodenname+": Nachricht ist vom Typ "+ersteNachricht.getClass().getName());
		} catch (IOException e) {
			KDCLogger.log(e);
		}
		return null;
	}
	
	public void close() throws IOException
	{
		this.outputStream.close();
		this.inputStream.close();
		this.socket.close();
	}
}
