/*
 * Created on 15.01.2004
 */
package util;

import java.io.IOException;
import java.io.InputStream;
import java.net.Socket;

import kerberos.exceptions.ArrayZuLangException;
import kerberos.exceptions.ConnectionResetException;
import kerberos.exceptions.KerberosRuntimeException;
import kerberos.messages.MessageBuilder;
import kerberos.net.Passwort;
import kerberos.packets.PacketException;

/**
 * Diese Klasse fasst einige Netzwerk-Utilities zusammen
 * @author Stephan Koops
 */
public class NetUtil
{
	public static final boolean DEBUG = false;

	/** Keine Objekterzeugung mglich */
	private NetUtil() { }
	
	/**
	 * Liest aus dem InputStream des Socket die nchsten 4 Bytes, baut daraus eine
	 * Integerzahl, interpretiert diesen Wert als Lnge und liest entsprechend viele Zeichen aus.<br>
	 * Wenn die Lnge des zu bildenden Arrays die maximal erlaubte Lnge des Arrays berschreitet, wird 
	 * eine entsprechende Fehlermeldung zurckgeschickt und auf die nchste Nachricht gewartet.
	 * @param socket
	 * @param maxLnge maximal erlaubte Lnge des Arrays (um OutOfMemoryErrors zu verhindern)
	 * @return byte-Array mit den gelesenen Zeichen oder null, wenn der Stream von Anfang an zu Ende ist.
	 * @throws ArrayZuLangException wird geworfen, wenn das zu lesende Array lnger ist als erlaubt
	 * @throws ConnectionResetException wenn der Stream zu Ende ist
	 * @throws IOException
	 * @see #writeArray(Socket, byte[])
	 */
	public static byte[] readByteArray(Socket socket, int maxLnge) throws ConnectionResetException, IOException
	{
		InputStream inputStream = socket.getInputStream();
		for(;;)
		{
			if (DEBUG)
			{
				System.out.println("NetUtil.readByteArray: Lnge auslesen:");
			} 
			int lnge = toInt(inputStream); // Die Anzahl der Zeichen, die vorgegeben werden, dass sie gelesen werden.
			boolean speichern = (lnge <= maxLnge); // mehr Zeichen als zugelassen?
			if (DEBUG)
			{
				System.out.println("NetUtil.readByteArray: Lnge ausgelesen: " + lnge);
			} 
			byte[] zeichen = null;
			System.gc();
			if(speichern)
				zeichen = new byte[lnge];
			for (int i = 0; i < lnge; i++)
			{
				int zahl = inputStream.read();
				if (DEBUG)
				{
//					System.out.println("KerberosSocket.readByteArray: " + zahl + " aus Stream ausgelesen");
				}
				if (zahl == -1)
					throw new ConnectionResetException();
				if (speichern) // nur speichern, wenn auch ausgelesen wird
					zeichen[i] = (byte)zahl;
			}
			if (speichern) // Nachrichtenlnge OK?
			{ 
				if (DEBUG)
				{
					System.out.println("NetUtil.readByteArray: gelesen: "+Passwort.toZahl(zeichen));
				}
				return zeichen;
			}
			if(DEBUG)
			{
				System.out.println("NetUtil.readByteArray: Es sollen " + lnge+" Zeichen ausgelesen werden, aber das ist zuviel (weil > "+maxLnge+")");
			} 
			ArrayZuLangException.sendError(socket);
			// neuer Versuch
		}
	}
	
	/**
	 * Wandelt die bergebene int-Zahl in ein Byte-Array um
	 */
	public static byte[] toByteArray(int l)
	{
		byte byte4[] = new byte[4];
		for (int i = 3; i >= 0; i--)
			byte4[i] = (byte) ((int) (l >> (3 * 8 - i * 8) & 255L));
		return byte4;
	}
	
	/**
	 * Liest 4 Zeichen aus dem Stream aus und interpretiert diese als int-Zahl
	 * @throws ConnectionResetException
	 * @throws IOException
	 */
	public static int toInt(InputStream byteStream) throws ConnectionResetException, IOException
	{
		int result = 0;
		for (int i = 0; i <= 3; i++) // 0,1,2,3 = Typengre von int 
		{
			int zahl = byteStream.read();
			if (zahl == -1)
				throw new ConnectionResetException();
			result += zahl << (3 * 8 - i * 8);
		}
		if (DEBUG)
			System.out.println("NetUtil.toInt(InputStream): Lnge ist " + result);
		return result;
	}
	
	/**
	 * Schreibt das bergebene Array auf den OutputStream des Sockets,
	 * mit der Lngenangabe davor
	 * @see #toByteArray(int)
	 */
	public static void writeArray(Socket socket, byte[] array) throws IOException
	{
		// Lnge des Arrays in den Stream schreiben
		socket.getOutputStream().write(toByteArray(array.length));
		if(DEBUG)
		{
			System.out.println("NetUtil.writeArray: Lnge="+array.length);
		}
		// Daten schreiben
		socket.getOutputStream().write(array);
		if(DEBUG)
		{
			System.out.println("NetUtil.writeArray: "+Passwort.toZahl(array));
		}
		// flush
		if (DEBUG)
		{
			//System.out.println("NetUtil.writeArray: gleich stream.flush()");
		} 
		socket.getOutputStream().flush();
		if (DEBUG)
		{
			//System.out.println("NetUtil.writeArray: stream.flush() fertig");
		} 
	}
	
	/**
	 * Sendet eine Fehlermeldung ins Netz. Eine evtl auftretende IOException
	 * wird ignoriert.
	 * @param socket
	 * @param messageType
	 * @param errorcode
	 * @throws KerberosRuntimeException Wird geworfen, wenn eine PacketException aufgetreten ist
	 * @see MessageBuilder#buildError(int, int)
	 * @see PacketException
	 */
	public static void sendError(Socket socket, int messageType, int errorcode) throws KerberosRuntimeException
	{
		try
		{
			sendPaket(socket, MessageBuilder.currentBuilder().buildError(messageType, errorcode));
		}
		catch (PacketException pe)
		{
			throw KerberosRuntimeException.throwWith(pe);
		} 
		catch (IOException e)
		{
		}
	}
	
	public static void sendPaket(Socket socket, byte[] paket) throws IOException
	{
		NetUtil.writeArray(socket, paket);
	}
}
